package com.sharppaint;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.*;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.*;

import com.sharppaint.MainWindow;
import com.sharppaint.drawtools.DrawToolConst;
import com.sharppaint.drawtools.Element;
import com.sharppaint.drawtools.EllipseTool;
import com.sharppaint.drawtools.GradientTool;
import com.sharppaint.drawtools.ImageTool;
import com.sharppaint.drawtools.Layer;
import com.sharppaint.drawtools.LineTool;
import com.sharppaint.drawtools.MultiLineTool;
import com.sharppaint.drawtools.PenTool;
import com.sharppaint.drawtools.PointF;
import com.sharppaint.drawtools.RectangleTool;
import com.sharppaint.drawtools.RoundRectTool;
import com.sharppaint.drawtools.TextTool;
import com.sharppaint.file.FileTool;
import com.sharppaint.toolbar.UpperToolBox;
import com.sharppaint.toolbar.WestToolBox;



import java.awt.image.VolatileImage;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.util.*;

//setFillColor
public class DrawingPanel extends JPanel 
			implements KeyListener, Scrollable, Printable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 6402239579932776081L;
	
	//constants
	public final static int MULTILINE=1;
	public final static int LINE=2;
	public final static int RECT=3;
	public final static int ROUNDRECT=4;
	public final static int ELLIPSE=5;
	public final static int TEXT=6;
	public final static int TEXT_EDIT_BOX=7;
	public final static int PEN=8;
	public final static int LAYER=10;
	public final static int IMAGE=11;
	public final static int MOVE=12;
	public final static int COPY=13;
	public final static int PAINT=14;
	public final static int RESIZE=15;
	public final static int PREVIEW=22;
	public final static int PEN_START=30;
	public final static int PEN_WHILE=31;
	public final static int RESHAPE = 33;
	
	private Vector<Element> drawVector;
	private Vector<Element> redoVector;
	//undo, redo
	private ArrayList<Boolean>  undoAddFlagList, redoRemoveFlagList;
	
	private Element element;
	private Element copyElement;
	private Element textTempElement;
	private Element tempElement;	
	private Element drawElement;
	// ġ elementġ ġ 
	private boolean isElementMatch;
	//element index
	private int currentElementIndex;
	private int tempElementIndex;
	private int tempTextElementIndex;

	//buffer image
	private VolatileImage vimg;
	private VolatileImage vimgUsingDrag;
	//coordinate
	private float x1=0, x2=0, y1=0, y2=0;
	//color
	private Color strokeColor;
	private Color fillColor;
	//font
	private Font font;
	private int fontSize;
	private int fontStyle;
	private String fontName="";
	//file
	private String fileName, filePath;

	//original panel size
	private float panelWidth, panelHeight;
	//panel size
	private float width, height;
	//flags
	private boolean saved, closable, ctrlPush, ctrlContinue, shiftPush;
	private boolean textBoxDrawingMode, textModifyExistElement, textModifyElement;
	//quality flags
	private static boolean antiMode, renderMode;
	private static boolean ditheringMode, textAntiMode;
	private static boolean alphaInterpMode, colorRenderMode;
	private static boolean interpolationMode;
	//scale printing 
	private float scalePrint;
	private boolean scalePrintFlag;
	//style
	private boolean colorMode;
	private boolean gradientMode;
	private boolean patternMode;
	private boolean strokeMode;
	//gradient
	private float scaleGrad, angleGrad;
	private String patternPath, strokePatternPath;		
	private int patternScale;							
	private int gradientStyle;
	private int strokeGradientStyle;
	private boolean isGradientCycle;
	private boolean isStrokeGradientCycle;
	private ArrayList<GradientTool> currentGradList;
	//stroke
	private float strokeSize;
	private int strokeFillSelector;
	private int strokePosition;
	private int strokePatternScale;
	private float strokeScaleGrad;
	private float strokeAngleGrad;
	private ArrayList<GradientTool> currentStrokeGradList;
	//opacity
	private int colorOpacity, gradientOpacity, patternOpacity, strokeOpacity;
	//zoom
	private float canvasZoomValue;
	private boolean isCanvasZoom;
	//private boolean isZoomingMode;
	//tool
	private int drawingTool;
	//Text
	private JTextPane textBox;
	//roundRect
	private float roundRectArcWidth;
	private float roundRectArcHeight;
	//pen
	private boolean penToolClose = false;	
	//pen undo
	private ArrayList<PointF> undoPenList;
	
	//layer Number
	protected int layerNum;
	//element Number in Layer
	protected int elementNum;
	//parent Layer Number
	protected int parentLayerNum;
	//parent Layer Depth
	protected int parentLayerDepth;
	//is layer Select
	boolean layerSelection;
	
	private boolean isRepaintAll;
	
	//  (drawVector, 콺 ̺Ʈ ,  )
	public DrawingPanel(Dimension d) {
		super();
		this.setPreferredSize(d);
		this.setMinimumSize(d);
		panelWidth=(float)d.getWidth();
		panelHeight=(float)d.getHeight();
		setBackground(Color.white);
		drawVector = new Vector<Element>();
		redoVector = new Vector<Element>();
		undoAddFlagList = new ArrayList<Boolean>();
		redoRemoveFlagList = new ArrayList<Boolean>();
		addMouseListener( new MouseHandler() );
		addMouseMotionListener( new MouseMotionHandler() );
		addKeyListener(this);
		//currentElementIndex = 0;
		fillColor = Color.BLACK;
		strokeColor = Color.BLACK;
		strokeSize = 1.0f;
		colorOpacity = 100;
		strokeOpacity = 100;
		strokeFillSelector = DrawToolConst.STROKE_COLOR;
		strokePosition = DrawToolConst.STROKE_POSITION_CENTER;
		strokeAngleGrad = 0;
		if(drawingTool==0) {
			drawingTool = RECT;
		}
		strokeMode = true;
		antiMode = true;
		//renderMode = true;
		this.setLayout(null);
		this.setDoubleBuffered(true);
		fontSize = 18;
		fontStyle=0;
		isCanvasZoom = false;
		scalePrintFlag = false;
		scalePrint = 1.0f;
		canvasZoomValue = 1.0f;
		roundRectArcWidth = 10.0f;
		roundRectArcHeight = 10.0f;
		UpperToolBox.drawTool();
		//textBox = new JTextPane();
		//isRepaintAll = true;
		repaintAll();
	}
	
	public void initFirstLayer() {
		layerNum = 1;
		elementNum = 0;
		parentLayerNum = 0;
		parentLayerDepth = 0;
		//parentLayerDepth = 0;
		element = new Layer(layerNum, parentLayerNum);
		drawVector.add(element);
		layerSelection = true;
		currentElementIndex = 0;
		//MainWindow.getInstance().vectorLayerDialog.addVector(element, elementIndex);
		MainWindow.getInstance().vectorLayerDialog.initFirstLayer(element);
		MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
	}
	
	public Dimension getPanelSize() {
		return new Dimension((int)Math.round(panelWidth),
				(int)Math.round(panelHeight));
	}
	
	public void unDo(){
		if(undoAddFlagList.size() != 0) {
			//undo  redoÿ ߰ؾ  ̸
			if (!undoAddFlagList.get(undoAddFlagList.size()-1))  {
				if(drawVector.size() > 1){
					// Ҹ redoVector add
					redoRemoveFlagList.add(0, false);
					redoVector.add(0, drawVector.lastElement() );	
					drawVector.remove(drawVector.size()-1);
					undoAddFlagList.remove(undoAddFlagList.size()-1);
				}
			}
			//undo  redoÿ ؾ  ̸
			else if(redoVector.size() != 0){
				drawVector.add( redoVector.get(0) );
				redoRemoveFlagList.remove(0);				
				redoVector.remove(0);				
				undoAddFlagList.remove(undoAddFlagList.size()-1);
			}
			repaintAll(); 
			MainWindow.getInstance().vectorLayerDialog.setVector(drawVector);
			MainWindow.getInstance().setEnableRedoButton(true);
		}
		if(undoAddFlagList.size() == 0)
			MainWindow.getInstance().setEnableUndoButton(false);
	}
	
	public void reDo(){
		if (redoRemoveFlagList.size() != 0) {
			//redoÿ ߰ؾ  ̸
			if(!redoRemoveFlagList.get(0)) {
				if(!redoVector.isEmpty()){
					// ֽ  Ҹ drawVector add
					drawVector.add(redoVector.get(0));	
					redoVector.removeElementAt(0);
					redoRemoveFlagList.remove(0);
					//if ( undoAddFlagList.size() >0 )
					//	undoAddFlagList.set(undoAddFlagList.size()-1, false);					
					undoAddFlagList.add(false);
				}
			}
			//redoÿ  ؾߵ ̸
			else {
				drawVector.remove(drawVector.size()-1);
				redoVector.removeElementAt(0);
				redoRemoveFlagList.remove(0);				
				undoAddFlagList.add(true);
			}
			repaintAll();
			MainWindow.getInstance().vectorLayerDialog.setVector(drawVector);
		}
		if (redoRemoveFlagList.size() == 0)
			MainWindow.getInstance().setEnableRedoButton(false);

		MainWindow.getInstance().setEnableUndoButton(true);
	}
	
	//undo  redoÿ ߰ؾ  Ȳ  ok
	public void addUndoVector(Element element, int num) { 
		redoRemoveFlagList.add(0, false);
		redoVector.add(0, element);
		undoAddFlagList.add(true);
	}
	
	public void initCurrentElementInform() {
		currentElementIndex = drawVector.size()-1;	
		element = drawVector.get(currentElementIndex);
		layerNum = element.getLayerNum();
		elementNum = element.getElementNum();
		parentLayerNum = element.getParentLayerNum();
		parentLayerDepth = element.getParentLayerDepth();
		//LayerManagementDialog.getInstance().setBtnCompIndex(1);
		if(element.getToolName() == DrawToolConst.LAYER)
			layerSelection = true;
		else
			layerSelection = false;
		//System.out.println(layerSelection + "  layerSelection");
	}
	
	public void setSelectPanelColorInWestToolBar() {
		if(element.getColorMode()) {
			WestToolBox.getInstance().setFillColor(true, element.getFillColor());
		} else {
			WestToolBox.getInstance().setFillColorMode(false);
		}
		if(element.getStrokeMode() && 
				element.getStrokeFillMode() == DrawToolConst.STROKE_COLOR) {
			WestToolBox.getInstance().setStrokeColor(true, element.getStrokeColor());
		} else {
			WestToolBox.getInstance().setStrokeColorMode(false);
		}
	}
	
	public void addElement() {
		undoAddFlagList.add(false);
		element.setLayerNum(layerNum);
		element.setParentLayerNum(parentLayerNum);
		element.setParentLayerDepth(parentLayerDepth);
		if(layerSelection) {
			drawVector.add(currentElementIndex, element);
		}
		else {
			//System.out.println(elementIndex);
			drawVector.add(currentElementIndex+1, element);
		}
		MainWindow.getInstance().setEnableRedoButton(false);
		MainWindow.getInstance().setEnableUndoButton(true);
		MainWindow.getInstance().vectorLayerDialog.addVector(element);
		element.setSelected(false);
		//MainWindow.getInstance().vectorLayerDialog.foldingRealignment();
		MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
		//elementIndex++;
	}
	
	public void addFirstElement(Element e) {
		undoAddFlagList.add(false);
		element = e;
		element.setLayerNum(layerNum);
		element.setParentLayerNum(parentLayerNum);
		element.setParentLayerDepth(parentLayerDepth);
		if(layerSelection) {
			drawVector.add(currentElementIndex, element);
		}
		else {
			drawVector.add(currentElementIndex+1, element);
		}
		MainWindow.getInstance().setEnableRedoButton(false);
		MainWindow.getInstance().setEnableUndoButton(true);
		MainWindow.getInstance().vectorLayerDialog.addVector(element);
		MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
	}
	
	public void addCopyElement() {
		undoAddFlagList.add(false);
		copyElement.setLayerNum(layerNum);
		copyElement.setParentLayerNum(parentLayerNum);
		copyElement.setParentLayerDepth(parentLayerDepth);
		if(layerSelection) {
			//element.addElementNum();
			drawVector.add(currentElementIndex, copyElement);
		}
		else {
			drawVector.add(currentElementIndex+1, copyElement);
		}
		element = copyElement;
		//System.out.println("ElementIndex: " + elementIndex);
		MainWindow.getInstance().setEnableRedoButton(false);
		MainWindow.getInstance().setEnableUndoButton(true);
		MainWindow.getInstance().vectorLayerDialog.addVector(element);
		//MainWindow.getInstance().vectorLayerDialog.foldingRealignment();
		MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
		//elementIndex++;
	}
	
	public void setResize(float percent) {
		canvasZoomValue = percent;
		if(percent != 1.0) {
			isCanvasZoom = true;
		} else {
			isCanvasZoom = false;
		}
		this.setMinimumSize(
				new Dimension((int)(panelWidth*canvasZoomValue),
						(int)(panelHeight*canvasZoomValue)));
		this.setPreferredSize(
				new Dimension((int)(panelWidth*canvasZoomValue),
						(int)(panelHeight*canvasZoomValue)));
		//repaintAll();
		this.revalidate();
	}
	
	public void setSelect(Element e, int index) {
		for(int i=0; i<drawVector.size(); i++) {
			drawVector.get(i).setSelected(false);
		}
		element = e; 
		element.setSelected(true);
		currentElementIndex = index;
		drawVector.setElementAt(element, currentElementIndex);
		repaintAll();
    }
	
	public void setCurrentElementIndex(int elementIndex) {
		this.currentElementIndex = elementIndex;
	}
	
	public void setSelectDisable() {
		for(int i=0; i<drawVector.size(); i++) {
			drawVector.get(i).setSelected(false);
		} 
		repaintAll();
    }
	//Image Filters
	public void setGrayScale(Element _elem, int _elemIndex) {
		drawVector.setElementAt(_elem, _elemIndex);
		repaintAll();
	}
	
	public void setElement(Element e, int index) {
		drawVector.setElementAt(e, index);
    	repaintAll();
	}
	//edit
	public void setRotate(Element e, int index, int angle) {
    	e.setRotateAngle(angle);
    	drawVector.setElementAt(e, index);
    	repaintAll();
    }
	
	public ArrayList<GradientTool> getGradList() {
		return currentGradList;
	}
	
	public ArrayList<GradientTool> getStrokeGradList() {
		return currentStrokeGradList;
	}
	public void setBackupEdit(
			Element e, 
			int index, int scaleX, 
			int scaleY, int angle)	{
    	e.setScaleX(scaleX / 100.0);
    	e.setScaleY(scaleY / 100.0);
    	e.setRotateAngle(angle);
    	drawVector.setElementAt(e, index);
    	repaintAll();
    }
	public void setAntiMode(boolean _anti) {
		antiMode = _anti;
		repaintAll();
	}
	public void setRenderMode(boolean _render) {
		renderMode = _render;
		repaintAll();
	}
	public void setDitheringMode(boolean _ditheringMode) {
		ditheringMode = _ditheringMode;
		repaintAll();
	}
	public void setTextAntiMode(boolean _textAntiMode) {
		textAntiMode = _textAntiMode;
		repaintAll();
	}
	public void setAlphaInterpMode(boolean _alphaInterpMode) {
		alphaInterpMode = _alphaInterpMode;
		repaintAll();
	}
	
	public void setColorRenderMode(boolean _colorRenderMode) {
		colorRenderMode = _colorRenderMode;
		repaintAll();
	}
	
	public void setInterpolationMode(boolean _interpolationMode) {
		interpolationMode = _interpolationMode;
		repaintAll();
	}
	public void setResizeX(Element e, int index, int scaleX) {
    	e.setScaleX(scaleX / 100.0);
    	drawVector.setElementAt(e, index);
    	repaintAll();
    }
	
	public void setResizeY(Element e, int index, int scaleY) {
    	e.setScaleY(scaleY / 100.0);
    	drawVector.setElementAt(e, index);
    	repaintAll();
    }
	
	public void setGradOpacity(Element e, int index, int _gradOpacity) {
		drawVector.setElementAt(e, index);
		gradientOpacity = _gradOpacity;
    	repaintAll();
	}
		
	public void setGradStyle(Element e, int index, int style) {
		drawVector.setElementAt(e, index);
		gradientStyle = style;		
    	repaintAll();
	}
	
	public void setGradCycle(Element e, int index, boolean cycle) {
		drawVector.setElementAt(e, index);
		isGradientCycle = cycle;		
    	repaintAll();
	}
	
	public void setGradient(Element _elem, int _elemIndex, 
			float _scaleGrad, float _angleGrad,  
			int _gradOpacity, boolean _gradientMode) {
		drawVector.setElementAt(_elem, _elemIndex);
		ArrayList<GradientTool> tempGradList = _elem.getGradList();		
		currentGradList = new ArrayList<GradientTool>();
		for(int i=0; i<tempGradList.size(); i++) {
			GradientTool gradRect = new GradientTool(tempGradList.get(i));
			currentGradList.add(gradRect);
		}
		gradientMode = _gradientMode;
		gradientOpacity = _gradOpacity;
		scaleGrad = _scaleGrad;
		angleGrad = _angleGrad;
		repaintAll();
	}
	
	public void setGradient(Element _elem, int _elemIndex, 
			boolean _gradientMode) {
		drawVector.setElementAt(_elem, _elemIndex);
		gradientMode = _gradientMode;
		repaintAll();
	}
	
	//pattern
	public void setPatternOpacity(Element e, int index, int _patternOpacity) {
		drawVector.setElementAt(e, index);
		patternOpacity = _patternOpacity;
    	repaintAll();
	}
	
	public void setPattern(
			Element _elem, int _elemIndex, String _patternPath, 
			int _patternScale, int _patternOpacity, boolean _patternMode) {
		drawVector.setElementAt(_elem, _elemIndex);
		patternMode = _patternMode;
		patternPath = _patternPath;
		patternScale = _patternScale;	
		patternOpacity = _patternOpacity;
		repaintAll();
	}
	
	public void setPattern(Element _elem, int _elemIndex, 
			boolean _patternMode) {
		drawVector.setElementAt(_elem, _elemIndex);
		patternMode = _patternMode;
		repaintAll();
	}
	
	public void setPatternScale(Element _elem, 
			int _elemIndex, int _patternScale) {
		drawVector.setElementAt(_elem, _elemIndex);
		patternScale = _patternScale;
		repaintAll();
	}
	
	//stroke
	public void setStrokeGradStyle(Element e, int index, int style) {
		drawVector.setElementAt(e, index);
		strokeGradientStyle = style;	
    	repaintAll();
	}
	
	public void setStrokeGradCycle(Element e, int index, boolean cycle) {
		drawVector.setElementAt(e, index);
		isStrokeGradientCycle = cycle;		
    	repaintAll();
	}
	
	public void setStrokePattern(Element _elem, int _elemIndex, 
			String _patternPath, int _patternScale) {
		drawVector.setElementAt(_elem, _elemIndex);
		strokePatternPath = _patternPath;
		strokePatternScale = _patternScale;	
		strokeFillSelector = 2;
		repaintAll();
	}
	
	public void setStrokePatternScale(
			Element _elem, int _elemIndex,  int _patternScale) {
		drawVector.setElementAt(_elem, _elemIndex);
		strokePatternScale = _patternScale;	
		repaintAll();
	}
	
	@SuppressWarnings("unchecked")
	public void setStrokeGradient(Element _elem, int _elemIndex, 
			float _scaleGrad, float _angleGrad) {
		currentStrokeGradList = 
			(ArrayList<GradientTool>)_elem.getStrokeGradList().clone();
		if(drawVector.size() > _elemIndex)
			drawVector.setElementAt(_elem, _elemIndex);
		strokeFillSelector = 1;
		strokeScaleGrad = _scaleGrad;
		strokeAngleGrad = _angleGrad;
		repaintAll();
	}
	
	public void setFuncStrokeSize(
			Element _elem, int _elemIndex, float _strokeSize) {
		drawVector.setElementAt(_elem, _elemIndex);
		strokeSize = _strokeSize;
		repaintAll();
	}
	
	public void setStrokeOpacity(Element e, int index, int _strokeOpacity) {
		drawVector.setElementAt(e, index);
		strokeOpacity = _strokeOpacity;
    	repaintAll();
	}
	
	public void setStrokePosition(Element e, int index, int _strokePosition) {
		drawVector.setElementAt(e, index);
		strokePosition = _strokePosition;
    	repaintAll();
	}
	
	public void setStrokeFillMode(Element e, int index, int _strokeFillMode) {
		drawVector.setElementAt(e, index);
		strokeFillSelector = _strokeFillMode;
    	repaintAll();
	}
	
	public void setStrokeColor(Element e, int index, Color _strokeColor) {
		drawVector.setElementAt(e, index);
		strokeColor = _strokeColor;
    	repaintAll();		
	}
	
	public Color getStrokeColor() {
		return strokeColor;
	}
	
	public void setStroke(Element e, int index,
			float _strokeSize, int _strokeOpacity,
			int _strokePosition, int _strokeFillMode, boolean _strokeMode) {
		drawVector.setElementAt(e, index);
		strokeSize = _strokeSize;
		strokeOpacity = _strokeOpacity;
		strokePosition = _strokePosition;
		strokeFillSelector = _strokeFillMode;
		strokeMode = _strokeMode;
    	repaintAll();
	}
	
	public void setStroke(Element e, int index, boolean _strokeMode) {
		drawVector.setElementAt(e, index);
		strokeMode = _strokeMode;
    	repaintAll();
	}
	//color
	public void setFillColor(Element e, int index, Color _overlayColor, 
			int _colorOpacity, boolean _colorMode) {
		drawVector.setElementAt(e, index);
		fillColor = _overlayColor;
		colorOpacity = _colorOpacity;
		colorMode = _colorMode;
    	repaintAll();
	}
	
	public void setFillColor(Element e, int index, boolean _colorMode) {
		drawVector.setElementAt(e, index);
		colorMode = _colorMode;
    	repaintAll();
	}
	
	public void setColorOpacity(Element e, int index, int _colorOpacity) {
		drawVector.setElementAt(e, index);
		colorOpacity = _colorOpacity;
    	repaintAll();
	}
	
	public Color getFillColor() {
		return fillColor;
	}
	
	public void setHide(Element e, boolean _hide, int index) {
    	e.setHide(_hide);
    	drawVector.setElementAt(e, index);
    	repaintAll();
    }
	
	public void setSaved() {
		saved = true;
	}
	
	public boolean getSaved() {	
		return saved;
	}
	
	public void setClosable(boolean _close) {
		closable = _close;
	}
	
	public boolean getClosable() {
		return closable;
	}
	
	public void setFileName(String name) {
		fileName = name;
	}
	
	public void setFilePath(String path) {
		filePath = path;
	}
	
	public String getFileName() {
		return fileName;
	}
	
	public String getFilePath() {
		return filePath;
	}
	
	public void setVector(Vector<Element> vector) { drawVector = vector; }
	
	public Vector getVector(){return drawVector;}
	
	public void setDrawingTool(int _tool) {	//  ( ToolBar )
		if(drawingTool == PEN_WHILE && element.getToolName() == PEN) {
			((PenTool)element).setPenBounds();
			((PenTool)element).setPenWhileMode(false);
		}
		drawingTool = _tool;
		if(drawingTool == MOVE)
			setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
		else if(drawingTool == RESIZE)
			setCursor(Cursor.getPredefinedCursor(Cursor.NW_RESIZE_CURSOR));
		else if(drawingTool == COPY || drawingTool == PAINT)
			setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		//else if(tool == POINT)
		//	alreadyPoint = false;
		else
			setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
	
	public void setColorMode(boolean _fillMode) {
		colorMode = _fillMode;
		if(layerSelection) {
			//drawVector.get(elementIndex-1).setColorMode(colorMode);
		}
		else {
			drawVector.get(currentElementIndex).setColorMode(colorMode);
		}
		MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
		repaintAll();
	}
	
	public void setStrokeMode(boolean _strokeMode) {
		strokeMode = _strokeMode;
		if(layerSelection) {
			//drawVector.get(elementIndex-1).setStrokeMode(strokeMode);
		}
		else {
			drawVector.get(currentElementIndex).setStrokeMode(strokeMode);
		}
		MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
		repaintAll();
	}
	
	public void setFillColor(Color color) {	//  ( ToolBar )
		fillColor = color;
		colorMode = true;
		if(layerSelection) {
			//drawVector.get(elementIndex-1).setFillColor(fillColor);
		}
		else {
			drawVector.get(currentElementIndex).setFillColor(fillColor);
			MainWindow.getInstance().vectorLayerDialog.setStyle(
					currentElementIndex);
			MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
		}
		//System.out.println(elementIndex);
		repaintAll();
	}
	
	public void setStrokeColor(Color color) {	//  ( ToolBar )
		strokeColor = color;
		strokeMode = true;
		//if(tool == MOVE) {
		if(layerSelection) {
			//drawVector.get(elementIndex-1).setStrokeColor(strokeColor);
		}
		else {
			drawVector.get(currentElementIndex).setStrokeColor(strokeColor);
			drawVector.get(currentElementIndex).setStrokeSize(strokeSize);
			MainWindow.getInstance().vectorLayerDialog.setStyle(
					currentElementIndex);
			MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
		}
		//}
		repaintAll();
	}
	
	public void setFontSize(int fsize) {//Ʈ ũ  ( ToolBar )
		fontSize = fsize;
		if(textBox != null) {
			font = new Font(fontName, fontStyle, fontSize);
			textBox.setFont(font);
			setFocusToTextArea();
		}
	}
	
	public void setFontStyle(int fstyle) {//Ʈ Ÿ  ( ToolBar )
		fontStyle = fstyle;
		if(textBox != null) {
			font = new Font(fontName, fontStyle, fontSize);
			textBox.setFont(font);
			setFocusToTextArea();
		}
	}
	
	public void setFontName(String fname) {//Ʈ  ( ToolBar )
		fontName = fname;
		if(textBox != null) {
			font = new Font(fontName, fontStyle, fontSize);
			textBox.setFont(font);
			setFocusToTextArea();
		}
	}
	
	public void setStrokeSize(float _strokeSize) {	//set Stroke Size
		strokeSize = _strokeSize;
		if(layerSelection) {
		}
		else {
			if(drawVector.size()>currentElementIndex)
				drawVector.get(currentElementIndex).setStrokeSize(strokeSize);
		}
		MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
		repaintAll();
	}
	
	//create Image to save
	private RenderedImage myCreateImage(int _width, int _height) {	
		int width = _width;
		int height = _height;	    
	    // Create a buffered image in which to draw
		BufferedImage bufferedImage = new BufferedImage(
				width, height, BufferedImage.TYPE_INT_RGB);
	    // Create a graphics contents on the buffered image
		Graphics2D g2d = bufferedImage.createGraphics();
		g2d.setColor(Color.WHITE);
		g2d.fillRect(0, 0, width, height);
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		g2d.setRenderingHint(RenderingHints.KEY_RENDERING,
				RenderingHints.VALUE_RENDER_QUALITY);
		for(int i=0; i<drawVector.size(); i++) {  // Draw graphics
			drawElement = drawVector.elementAt(i);
			drawElement.draw(g2d);
		}			    
		g2d.dispose();
		return bufferedImage;
	}
	
	public void save(String fileName, int _width, int _height) {	//image save
		// Create an image to save	   	
    	String []s=fileName.split("[.]");
    	String ext = s[1];
	    RenderedImage rendImage = myCreateImage(_width, _height);	
	    try {	// Write generated image to a file	        
	        File file = new File(fileName);	// Save as jpg
	        if(ext.equals("jpg") || ext.equals("JPG")) {
	        	ImageIO.write(rendImage, "jpg", file);
	        } else if(ext.equals("gif") || ext.equals("GIF")) {
	        	ImageIO.write(rendImage, "jpg", file);
	        } else if(ext.equals("png") || ext.equals("PNG")) {
	        	ImageIO.write(rendImage, "png", file);
	        }
	    } catch (IOException e) {	    }
	}
	
	private void copyElementSetting() {
		copyElement.setX1( element.getX2()-element.getWidth() );
		copyElement.setX2( element.getX2() ); 
		copyElement.setWidth( element.getWidth() );
		copyElement.setY1( element.getY2()-element.getHeight() );
		copyElement.setY2( element.getY2() ); 
		copyElement.setHeight( element.getHeight() );
	}
	
	public void setFocusToTextArea() {
		if(textBox != null) {
			//textBox.invalidate();
			//textBox.validate();
			textBox.repaint();
			if(!textBox.isFocusOwner()) {
				textBox.requestFocusInWindow();
			}
		}
	}
	
	
	private void elementModeSetting() {
		if(colorMode) {
			element.setFillColor(fillColor, colorOpacity, true);
		}
		if(gradientMode) {
			element.setGradient(currentGradList, gradientOpacity, true);
			element.setGradVar(scaleGrad, angleGrad);
			element.setGradStyle(gradientStyle);
			element.setGradCycle(isGradientCycle);
		}
		if(patternMode) {
			element.setPatternImage(patternPath, patternScale, 
					patternOpacity, true);
		}
		if(strokeMode) {
			element.setStroke(strokeSize,  
					strokeOpacity, strokePosition, 
					strokeFillSelector, true);
			if(strokeFillSelector == DrawToolConst.STROKE_COLOR) {	//stroke 
				element.setStrokeColor(strokeColor);
			}
			else if(strokeFillSelector == DrawToolConst.STROKE_GRADIENT) {	//stroke gradient
				element.setStrokeGradient(currentStrokeGradList);
				element.setStrokeGradScale(strokeScaleGrad);
				element.setStrokeGradAngle(strokeAngleGrad);
				element.setStrokeGradStyle(strokeGradientStyle);
				element.setStrokeGradCycle(isStrokeGradientCycle);
			}
			else if(strokeFillSelector == DrawToolConst.STROKE_PATTERN) {	//stroke pattern
				element.setStrokePatternImage(strokePatternPath, 
						strokePatternScale); 
			}
		}
	}
	
	private void textPressed(float _x1, float _y1) {
		String inputText;
		float x1 = _x1;
		float y1 = _y1;

		for(tempElementIndex=drawVector.size()-1; tempElementIndex>=0 ; tempElementIndex--){ 
			element = drawVector.elementAt(tempElementIndex);
			isElementMatch = element.match(x1, y1);
			if(isElementMatch == true) {
				if(element.getToolName() == TEXT) {
					tempTextElementIndex = tempElementIndex;
					textTempElement = element;
					break;
				}
				else
					isElementMatch = false;
			}
		}

		// TextTool  ǥ  
		if (isElementMatch == true) {
			if(!textBoxDrawingMode) { //&& !textModifyElement && !textModifyExistElement
				if(textModifyExistElement) {
					textBox.setVisible(false);
					textBox = null;
				}
				if(textModifyElement) {
					inputText = null;
					inputText = textBox.getText();
					if(inputText != null && !inputText.equals("")) {
						x1 = (float)textBox.getLocation().getX();
						y1 = (float)textBox.getLocation().getY();
						element = new TextTool(
								x1, y1, fontSize,
								fontStyle, fontName, inputText);
						((TextTool)element).setTextEditBoxWidth(textBox.getWidth());
						((TextTool)element).setTextEditBoxHeight(textBox.getHeight());
						((TextTool)element).initTextTool();
					    
						elementModeSetting();
						addElement();
						element = textTempElement;
					}
					textBox.setVisible(false);
					textBox = null;
					textModifyElement = false;
					//repaintAll();
				}
				x1 = element.getX1();
				y1 = element.getY1();
				font = ((TextTool)element).getFont();
				textBox = new JTextPane();
				textBox.setFont(font);
				textBox.setLocation(
						Math.round(x1), 
						Math.round(y1));
				textBox.setSize(
						Math.round(((TextTool)element).getTextEditBoxWidth()),
						Math.round(((TextTool)element).getTextEditBoxHeight()));
				textBox.setText(((TextTool)element).getText());
				textBox.setBorder(
						BorderFactory.createLineBorder(Color.RED));
				add(textBox);
				//textpane.selectAll();
				textModifyExistElement = true;	
				//textBox.revalidate();
				//textBox.setIgnoreRepaint(true);
				textBox.setVisible(true);
				textBox.requestFocusInWindow();
				//System.out.println("!textBoxDrawingMode");
			}
		}
		// TextTool  ǥ  textBox mode
		else {
			// 
			if(textModifyExistElement) {
				inputText = textBox.getText();
				if(inputText != null && !inputText.equals("")) {
					font = new Font(fontName, fontStyle, fontSize);
					tempElementIndex = tempTextElementIndex;
					element = drawVector.elementAt(tempElementIndex);
					((TextTool)element).setFont(
							fontName, fontStyle, fontSize);
					((TextTool)element).setText(inputText);
				    ((TextTool)element).initTextTool();
					/*
					Graphics g2 = getGraphics();
					FontMetrics metrics = 
						g2.getFontMetrics(((TextTool)element).getFont());
				    float adv = metrics.stringWidth(inputText);
				    float hgt = metrics.getHeight() - (metrics.getHeight()-fontSize)/2.0f ;*/
				    
					drawVector.setElementAt(element, tempElementIndex);
					textModifyExistElement = false;
					textBox.setVisible(false);
					textBox = null;
					//System.out.println("textModifyExistElement");
				}
			}
			else {
				if(!textBoxDrawingMode && !textModifyElement) {
					element = new RectangleTool(x1, y1);
					element.setStroke(1, 100, 
							 DrawToolConst.STROKE_POSITION_CENTER,
							 DrawToolConst.STROKE_COLOR, true);
					element.setStrokeColor(Color.blue);
					element.setColorMode(false);
					
					((RectangleTool)element).setTextShape();
					drawVector.add(element);
					textBoxDrawingMode = true;
					//System.out.println("!textBoxDrawingMode && !textModifyElement");
				}
			}
		}				
	}
	
	//콺 ۿ   ̺Ʈ ó
	private class MouseHandler extends MouseAdapter {
		public void mousePressed(MouseEvent e){
			closable=false;	//Ŭ window ׳    
			x1 = e.getX();
			y1 = e.getY();
			isElementMatch=false;
	    	if(isCanvasZoom) {
	    		x1 = Math.round((float)x1 / canvasZoomValue);
	    		y1 = Math.round((float)y1 / canvasZoomValue);
	    	}
			if(element == null) { }
			else if(ctrlPush==true) {
				for(tempElementIndex=drawVector.size()-1; tempElementIndex>=0 ; tempElementIndex--){ 
					tempElement = drawVector.elementAt(tempElementIndex);
					isElementMatch = tempElement.match(x1, y1);
					if(isElementMatch == true) {
						break;
					}
				}
				if(drawingTool == PEN_WHILE && element.getToolName() == PEN) {
					((PenTool)element).setPenWhileMode(false);
					//penToolClose = true;
					drawingTool = PEN;
				} else if (isElementMatch == true) {
					element = tempElement;
					setSelectPanelColorInWestToolBar();
					MainWindow.getInstance().vectorLayerDialog.setLayerSelectByElemNum(
							tempElementIndex);
					MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();	
					ctrlContinue = true;
				} 
				//else
				//	element = null;
			}
			else if(drawingTool == PEN) {	//multiLine
				element = new PenTool(x1,y1);
				((PenTool)element).setPenWhileMode(true);
				elementModeSetting();
				addElement();
			}
			else if(drawingTool == PEN_WHILE && element.getToolName() == PEN) {
				((PenTool)element).setDestPoint(x1, y1);

				if(((PenTool)element).isStartPoint(x1, y1)) {
					penToolClose = true;
				}
			}
			else if(drawingTool == MULTILINE) {	//multiLine
				element = new MultiLineTool(x1, y1);
				//mode Setting
				elementModeSetting();
				addElement();
			}
			else if(drawingTool == LINE) {
				element = new LineTool(x1, y1);
				element.setValue(x1, y1, x1, y1);
				//element = new LineTool(x1, y1, strokeSize, backColor);
				elementModeSetting();
				addElement();
			}
			else if(drawingTool == RECT) {
				element = new RectangleTool(x1, y1);
				//mode Setting
				elementModeSetting();
				addElement();	
			}
			else if(drawingTool == ELLIPSE) {
				element = new EllipseTool(x1, y1);	
				//mode Setting
				elementModeSetting();
				addElement();		
			}
			else if(drawingTool == ROUNDRECT) {
				element = new RoundRectTool(x1, y1);
				((RoundRectTool)element).setArcWidthHeight(
						roundRectArcWidth,
						roundRectArcHeight);
				//mode Setting
				elementModeSetting();
				addElement();	
			}
			else if(drawingTool == TEXT) {
				textPressed(x1, y1);
			}
			else if(drawingTool == IMAGE) {
				element = new ImageTool(x1, y1);
				addElement();
			}
			else if(drawingTool == RESHAPE) {
				if(element.reShapeMatch(e.getX(), e.getY())) {
					setCursor(Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR));
				} else {
					element.setSelected(false);
					for(tempElementIndex=drawVector.size()-1; tempElementIndex>=0 ; tempElementIndex--){ 
						element = drawVector.elementAt(tempElementIndex);
						//element.setSelected(false);
						isElementMatch = element.match(x1, y1);
						if(isElementMatch == true) {
							break;
						}
					}
					if(isElementMatch == true) {
						setSelectPanelColorInWestToolBar();
						MainWindow.getInstance().vectorLayerDialog.setLayerSelectByElemNum(
								tempElementIndex);
						MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
						//element.setSelected(true);
					} else {
						element = null;
					}
					drawingTool = MOVE;
				}
			}
			else if(drawingTool == MOVE || drawingTool == PAINT || drawingTool == RESIZE) {
				for(tempElementIndex=drawVector.size()-1; tempElementIndex>=0 ; tempElementIndex--){ 
					tempElement = drawVector.elementAt(tempElementIndex);
					isElementMatch = tempElement.match(x1, y1);
					if(isElementMatch == true) {
						//currentElementIndex = moveIndex;
						break;
					}
				}
				if (isElementMatch == true) {
					//move tool
					if (drawingTool == MOVE) {
						element = tempElement;
						if(element.getToolName() == DrawToolConst.ROUNDRECT) {
							UpperToolBox.drawRoundRectTool();
						} else if(element.getToolName() == DrawToolConst.TEXT) {
							UpperToolBox.textTool();
						} else {
							UpperToolBox.drawTool();
						}
						setSelectPanelColorInWestToolBar();
						MainWindow.getInstance().vectorLayerDialog.setLayerSelectByElemNum(
								tempElementIndex);
					}
					MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
					//setSelect layer
				}
				else
					element = null;
			}
			else if(drawingTool == COPY) {
				for(tempElementIndex=drawVector.size()-1; tempElementIndex>=0 ; tempElementIndex--){ 
					element = drawVector.elementAt(tempElementIndex);
					isElementMatch = element.match(x1, y1);
					if(isElementMatch == true) {
						int toolshape = element.getToolName();	
						if(toolshape == DrawToolConst.RECT) {
							copyElement = new RectangleTool(element);
							copyElementSetting();
						}
						else if(toolshape == DrawToolConst.LINE)
							copyElement = new LineTool(element);
						else if(toolshape == DrawToolConst.PEN)
							copyElement = new PenTool(element);
						else if(toolshape == DrawToolConst.MULTILINE)
							copyElement = new MultiLineTool(element);
						else if(toolshape == DrawToolConst.ELLIPSE) {
							copyElement = new EllipseTool(element);
							copyElementSetting();
						}
						else if(toolshape == DrawToolConst.ROUNDRECT) {
							copyElement = new RoundRectTool(element);
							copyElementSetting();
						}
						else if(toolshape == DrawToolConst.IMAGE) {
							copyElement = new ImageTool(element);
							copyElementSetting();
						}
						else if(toolshape == DrawToolConst.TEXT) {
							copyElement = new TextTool(element);
						}
						setSelectPanelColorInWestToolBar();
						if(copyElement != null) {
							addCopyElement();
							break;
						}							
					}
				}
				if (isElementMatch != true)
					element = null;
			}
			//repaintAll(); 9/14 fri ּ ó
	    	if(drawingTool != MOVE) {
	    	}
			else {
			}
		}
		
		public void mouseReleased(MouseEvent e){	//ư released
			x2 = e.getX();
			y2 = e.getY();
	    	if(isCanvasZoom) {
	    		x2 = Math.round((float)x2 / canvasZoomValue);
	    		y2 = Math.round((float)y2 / canvasZoomValue);
	    	}
			if(element == null) { }
			else if(ctrlContinue) { 
				ctrlContinue = false;					
				//vectorLayerDialog validate
				MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
			}
			else if(drawingTool == LINE){
				element.setValue(x1, y1, x2, y2);
				if(shiftPush)
					((LineTool)element).setEqualLine();
			}
			else if(drawingTool == RECT || drawingTool == ELLIPSE || drawingTool == ROUNDRECT) {
				element.setTransValue(x1, y1, x2, y2);
				if(shiftPush)
					element.setEqualWidthHeight();
			}
			else if(drawingTool == MULTILINE) {	//multiLine
				if(e.getButton() == 3)	//Ŭ path close
					((MultiLineTool)element).setMultiLineClose();
				((MultiLineTool)element).setMultiLineBounds();
				//((MultiLineTool)element).setPointfromPath();
			}
			else if(drawingTool == IMAGE) {				
				//jpg   MyImage  ׸.
				element.setTransValue(x1, y1, x2, y2);	
				boolean access = 
					FileTool.getInstance().openImage();
				if (access == true) {	
					((ImageTool)element).openImage(
							FileTool.getInstance().getFilePath());
				}				
			}
			else if(drawingTool == PEN) {	//multiLine	
				if(((PenTool)element).isPenWhileMode()) {
					if(!shiftPush) {
						((PenTool)element).setFirstCtrlValue(x2, y2);
					} else {
						((PenTool)element).setFirstCtrlValueEqual(x2, y2);						
					}
					drawingTool = PEN_WHILE;
				} 
			}
			else if(drawingTool == PEN_WHILE && element.getToolName() == PEN) {	//multiLine	
				if(penToolClose) {
					if(!shiftPush) {
						((PenTool)element).setSecondCtrlValue(x2, y2);
					} else {					
						((PenTool)element).setSecondCtrlValueEqual(x2, y2);
					}
					((PenTool)element).makeCurve();
					((PenTool)element).setPenBounds();
					((PenTool)element).setPenWhileMode(false);
					drawingTool = PEN;
					penToolClose = false;
				} else {
					if(!shiftPush) {
						((PenTool)element).setSecondCtrlValue(x2, y2);
					} else {
						((PenTool)element).setSecondCtrlValueEqual(x2, y2);
					}
					((PenTool)element).makeCurve();
				}
			}
			else if(drawingTool == MOVE || drawingTool == RESIZE) {	
				if(drawingTool == RESIZE) { 
					element.resize(x1, y1, x2, y2);
					if (element.getToolName() == DrawToolConst.LINE ) {
						((LineTool)element).setResizeDirectionFlag();
					}
					if(element.getToolName()  == DrawToolConst.IMAGE 
							&& e.getClickCount() == 2)
						((ImageTool)element).setOriginalImageScale();
				}
				if(drawingTool == MOVE) {
					if(element != null) {
						element.setReShape(true);
						drawingTool = RESHAPE;
					}
				}
				
				MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();		
			}

			
			/*
			else if(tool == RESHAPE) {
				if(element.reShapeMatch(e.getX(), e.getY())) {
					setCursor(Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR));
				}
				/*
				if(element != null && element.isReShape()) {
					if(element.reShapeMatch(e.getX(), e.getY())) {
						tool = RESHAPE;
					}
				}*/
			/*}
			if(element != null && element.isReShape()) {
				if(element.reShapeMatch(e.getX(), e.getY())) {
					tool = RESHAPE;
				}
			}*/
			else if(drawingTool == TEXT) {
				if(!textModifyExistElement) {
					if(textBoxDrawingMode) {
						//make textpane. text write enable
						if(drawVector.get(drawVector.size()-1).getToolName()
								== DrawToolConst.TEXT_EDIT_BOX)
							drawVector.remove(drawVector.size()-1);
						x1 = element.getX1();
						y1 = element.getY1();
						font = new Font(fontName, fontStyle, fontSize);
						textBox = new JTextPane();
						//textpane.setDoubleBuffered(true);
						textBox.setFont(font);
						textBox.setLocation(
								Math.round(x1), 
								Math.round(y1));
						textBox.setSize(
								Math.round(element.getWidth()),
								Math.round(element.getHeight()));
						textBox.setBorder(BorderFactory.createLineBorder(Color.BLUE));
						add(textBox);
						//textBox.selectAll();
						//textBox.revalidate();
						textBox.setVisible(true);
						textBox.requestFocusInWindow();
						//System.out.println("textBoxDrawingMode");
						textModifyElement = true;
						textBoxDrawingMode = false;
					} else if(textModifyElement) {
						//System.out.println("textModify");
						String inputText;
						inputText = textBox.getText();
						if(inputText != null && !inputText.equals("")) {
							x1 = (float)textBox.getLocation().getX();
							y1 = (float)textBox.getLocation().getY();
							//System.out.println("D x1:" + x1 + " y1:" + y1 + " x2:" + x2 + " y2:" + y2);
							element = new TextTool(
									x1, y1, fontSize, 
									fontStyle, fontName, inputText);
							((TextTool)element).setTextEditBoxWidth(textBox.getWidth());
							((TextTool)element).setTextEditBoxHeight(textBox.getHeight());
						    ((TextTool)element).initTextTool();
							
							/*Graphics g2 = getGraphics();
							FontMetrics metrics = 
								g2.getFontMetrics(((TextTool)element).getFont());
						    float adv = metrics.stringWidth(inputText);
						    float hgt = metrics.getHeight() - (metrics.getHeight()-fontSize)/2.0f;*/
						    
						    //System.out.println("t hgt:" + hgt + " adv:" + adv +  "e " + metrics.getHeight());
							//element = new TextTool(
							//		x1+1, y1+(fontSize*1.2f)+1, fontSize, 
							//		fontStyle, fontName, inputText);
							elementModeSetting();
							addElement();
						}
						textBox.setVisible(false);
						textBox = null;
						textModifyElement = false;
						//System.out.println("textModifyElement");
					}
				}
			}

			/*
			else if(tool == ERASER) {
				//eraserDrawMode = false;
				x1 = x2;
				y1 = y2;
				element.addEraser(new EraserTool(x1-10, y1-10, 20));
			}*/
			
			if(drawingTool != TEXT) {
				if(drawingTool != PEN_WHILE) {
		    		repaintAll();
		    		/*
			    	if(!isCanvasZoom) {
			    		repaint();
			    	} else {
			    		repaintAll();
			    	}*/
				} else {	//if pen while
					paintRecentElement();
					//repaintAll();
				}
				MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();		
			}
			else {	//if textTool
				if(textModifyElement) {					
				}
				else if(textModifyExistElement) {					
				}
				else {	//
					repaintAll();
					MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
				}
					
			}
		}
	}	
	
	private class MouseMotionHandler extends MouseMotionAdapter {
		//paintRecentElement();
		public void mouseDragged(MouseEvent e){
			x2 = e.getX();
			y2 = e.getY();
	    	if(isCanvasZoom) {
	    		x2 = x2 / canvasZoomValue;
	    		y2 = y2 / canvasZoomValue;
	    	}
			if(element != null) {
				if(ctrlContinue == true) {
					if(isElementMatch==true){
						element.translate(x1,y1,x2,y2);
						x1=x2;
						y1=y2;
						element.setFirstPatternMode();	//pattern ׷ֱ 
					}
				}
				else if(drawingTool == PEN) {
					if(!shiftPush) {
						((PenTool)element).setFirstCtrlValue(x2, y2);
					} else {
						((PenTool)element).setFirstCtrlValueEqual(x2, y2);						
					}					
				}
				else if(drawingTool == PEN_WHILE && element.getToolName() == PEN) {
					if(!shiftPush) {
						((PenTool)element).setSecondCtrlValue(x2, y2);
					} else {					
						((PenTool)element).setSecondCtrlValueEqual(x2, y2);
					}
				}
				else if(drawingTool == MULTILINE) {
					((MultiLineTool)element).setPathPoint(x2, y2);
					((MultiLineTool)element).setPoint(x2, y2);
				}
				else if(drawingTool == LINE) {
					element.setValue(x1, y1, x2, y2);
					if(shiftPush)
						((LineTool)element).setEqualLine();
				}
				else if(drawingTool == RECT) {
					element.setTransValue(x1, y1, x2, y2);
					if(shiftPush)
						element.setEqualWidthHeight();
				}
				else if(drawingTool == TEXT) {
					if(textBoxDrawingMode)
						element.setTransValue(x1, y1, x2, y2);
				}
				else if(drawingTool == ELLIPSE) {
					element.setTransValue(x1, y1, x2, y2);
					if(shiftPush)
						element.setEqualWidthHeight();
				}
				else if(drawingTool == ROUNDRECT) {
					element.setTransValue(x1, y1, x2, y2);
					if(shiftPush)
						element.setEqualWidthHeight();
				}
				else if(drawingTool == IMAGE) {
					element.setTransValue(x1, y1, x2, y2);
				}
				else if(drawingTool == RESIZE) {
					if(isElementMatch==true){
						element.resize(x1, y1, x2, y2);
						x1=x2;
						y1=y2;
					}
				}					
				else if(drawingTool == MOVE || drawingTool == COPY){
					if(isElementMatch==true){
						element.translate(x1,y1,x2,y2);	
						x1=x2;
						y1=y2;
						element.setFirstPatternMode();		
					}
				}
				else if(drawingTool == RESHAPE) {
					//if(element.reShapeMatch(e.getX(), e.getY())) {
					element.resize(x1,y1,x2,y2);
					x1=x2;
					y1=y2;
				}
				
				//if(tool != PEN_WHILE) {
				paintRecentElement();
				//} 
				//else {
				//	repaintAll();
				//}
			}
		}
		
		public void mouseMoved(MouseEvent e){
			//if not text modify then request focus (key focus)
			if(!textModifyElement && !textModifyExistElement) {
				requestFocusInWindow();
			}
			if(ctrlPush == true || (ctrlPush == false && ctrlContinue == true)){		
				for(tempElementIndex=drawVector.size()-1;
						tempElementIndex>=0 ; tempElementIndex--){ 
					tempElement = drawVector.elementAt(tempElementIndex);
					isElementMatch = tempElement.match(e.getX(), e.getY());
					if(isElementMatch == true) {
						break;
					}
				}
				if (isElementMatch==true) {
					//element = tempElement;
					setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				}
				else {
					ctrlContinue = false;
					setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
				}			
			}
			else if(drawingTool == PEN_WHILE) {
				if(element!= null && element.getToolName() == PEN) {					
					if(((PenTool)element).isStartPoint(e.getX(), e.getY())) {
						setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
					} else {
						setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					}
				}
			}
			else if(drawingTool == TEXT) {
				setFocusToTextArea();
			}

			else if(drawingTool == MOVE || drawingTool == COPY || drawingTool == RESIZE){
				for(tempElementIndex=drawVector.size()-1; 
						tempElementIndex>=0 ; tempElementIndex--){ 
					element = drawVector.elementAt(tempElementIndex);
					isElementMatch = element.match(e.getX(), e.getY());
					if(isElementMatch == true) {
						break;
					}
				}
				if (isElementMatch==true) {
					setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				}
				else {
					setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
				}
			}
			/*
			else if(tool == RESHAPE) {
				if(element.reShapeMatch(e.getX(), e.getY())) {
					setCursor(Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR));
				}
				/*
				if(element != null && element.isReShape()) {
					if(element.reShapeMatch(e.getX(), e.getY())) {
						tool = RESHAPE;
					}
				}*/
			/*}
			if(element != null && element.isReShape()) {
				if(element.reShapeMatch(e.getX(), e.getY())) {
					tool = RESHAPE;
				}
			}*/
			else if(drawingTool == RESHAPE) {
				if(element.reShapeMatch(e.getX(), e.getY())) {
					setCursor(Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR));
				} else {
					for(tempElementIndex=drawVector.size()-1; 
							tempElementIndex>=0 ; tempElementIndex--){ 
						tempElement = drawVector.elementAt(tempElementIndex);
						isElementMatch = tempElement.match(e.getX(), e.getY());
						if(isElementMatch == true) {
							break;
						}
					}
					if (isElementMatch==true) {
						setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
					}
					else {
						setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
					}
				}
			}
		}
	}
		
	public void keyTyped(KeyEvent e) {	
	}
	
	public void keyPressed(KeyEvent e) {
		if(e.getKeyCode() == KeyEvent.VK_CONTROL) {
			ctrlPush = true;
			//setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		} else if(e.getKeyCode() == KeyEvent.VK_SHIFT) {
			shiftPush = true;
		} else if(ctrlPush && shiftPush && e.getKeyCode() == KeyEvent.VK_Z) {
			if(undoPenList != null && drawingTool == PEN_WHILE
					&& element.getToolName() == PEN) {
				((PenTool)element).redoCurve();
				repaintAll();
				MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
			}
		} else if(ctrlPush && e.getKeyCode() == KeyEvent.VK_Z) {
			if(element != null && drawingTool == PEN_WHILE && element.getToolName() == PEN) {
				//System.out.println("pen");
				undoPenList = new ArrayList<PointF>();
				undoPenList = ((PenTool)element).removeCurve();
				if(undoPenList == null) {
					//drawVector.remove(elementIndex);
					MainWindow.getInstance().vectorLayerDialog.removeElement();
					//MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();	
					//element = null;
				}
				repaintAll();
			}
		} 
		/*
		if(e.getKeyCode() == KeyEvent.VK_7) {	if(ctrlPush) 
		{System.out.println("jjegrg");
				resizeScreen = resizeScreen + 0.2;
				at.scale(resizeScreen, resizeScreen);
				resize = true;
				repaint();
				//this.validate();
			}
		}*/
	}
	
	public void keyReleased(KeyEvent e) {
		if(e.getKeyCode() == KeyEvent.VK_CONTROL) {
			setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));	
			ctrlPush = false;
		}
		else if(e.getKeyCode() == KeyEvent.VK_SHIFT) {
			shiftPush = false;
		}
	}
	
	public void setScalePrint(float _scalePrint) {
		scalePrint = _scalePrint;
		if(_scalePrint==1.0f)
			scalePrintFlag = false;
		else
			scalePrintFlag = true;
	}
		
	float prevCanvasZoomValue = 1.0f;
	private void scaleSetting(Graphics2D g2d) {
    	if(isCanvasZoom) {
    		//g2d.scale(1.0f/prevCanvasZoomValue, 1.0f/prevCanvasZoomValue);
    		g2d.scale(canvasZoomValue, canvasZoomValue);
    		//prevCanvasZoomValue = canvasZoomValue;
    	}
	}
	//float prevZoomScreen=0.0f;
		
	private void qualitySetting(Graphics2D g2d) {
        if(antiMode) {
	        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
	               RenderingHints.VALUE_ANTIALIAS_ON);
        }
        if(renderMode) {
			g2d.setRenderingHint(RenderingHints.KEY_RENDERING,
					RenderingHints.VALUE_RENDER_QUALITY);
        }
        if(ditheringMode) {
        	g2d.setRenderingHint(RenderingHints.KEY_DITHERING,
					RenderingHints.VALUE_DITHER_ENABLE);
        }
        if(textAntiMode) {
        	g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
					RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        }
        if(alphaInterpMode) {
        	g2d.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION,
					RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
        }
        if(colorRenderMode) {
        	g2d.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING,
					RenderingHints.VALUE_COLOR_RENDER_QUALITY);
        }
        if(interpolationMode) {
        	g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
        			RenderingHints.VALUE_INTERPOLATION_BICUBIC);
        }
	}	
	
	/*
	public void repaint() {		
		if(!isZoomingMode) {
			//allRepaint = true;
			//allRepaint = false;
			super.repaint();
		} else {
			isRepaintAll = false;
			super.repaint();
		}		
	}*/
		
	public void repaintAll() {
		isRepaintAll = true;
		super.repaint();
	}
		
	private void paintRecentElement() {
		try {
    	Graphics2D g2 = (Graphics2D) getGraphics();
    	Graphics2D g2d = vimgUsingDrag.createGraphics();
        qualitySetting(g2d);
        if(!isCanvasZoom) {
        	//g2.scale(panelWidth/width, panelHeight/height);
        }
		g2d.drawImage(vimg, 0, 0, DrawingPanel.this);
		scaleSetting(g2d);
		if(element != null)
			element.draw(g2d);
		g2.drawImage(vimgUsingDrag, 0, 0, DrawingPanel.this);
		}catch(Exception e) {
			canvasZoomValue = 1.0f;
			
		}
	}
	
	public void paintComponent(Graphics g) {
		try {
		super.paintComponent(g);	//panel paintComponent
		if(isRepaintAll) {
			Graphics2D g2 = (Graphics2D)g;
			Graphics2D g2d = null;
			if(isCanvasZoom) {
				width = panelWidth*canvasZoomValue;
				height = panelHeight*canvasZoomValue;
			} else { 
				width = panelWidth;
				height = panelHeight;
			}
			if (vimg == null || vimg.getWidth() != width || vimg.getHeight() != height) {
				vimg = (VolatileImage)createVolatileImage(
						Math.round(width),
						Math.round(height));
			}
			g2d = vimg.createGraphics();
			g2d.setBackground(Color.WHITE);
			//g2d.setBackground(getBackground());
			g2d.clearRect(0, 0, 
					Math.round(width), 
					Math.round(height));
			qualitySetting(g2d);

			scaleSetting(g2d);
			for(int i=0; i<drawVector.size(); i++) {
				drawElement = drawVector.elementAt(i);
				drawElement.draw(g2d);
			}
			g2.drawImage(vimg, 0, 0, this);
			if (vimgUsingDrag == null || vimgUsingDrag.getWidth() != width
					|| vimgUsingDrag.getHeight() != height) {
				vimgUsingDrag = (VolatileImage)createVolatileImage(
						Math.round(width),
						Math.round(height));
			}
			g2d.dispose();
			g2.dispose();
			isRepaintAll = false;
			System.out.println("paintAll");
		} else {
			Graphics2D g2 = (Graphics2D)g;
			qualitySetting(g2);
			//if(isCanvasZoom) {
				g2.scale(panelWidth/width, panelHeight/height);
			//}
			scaleSetting(g2);
			g2.drawImage(vimg, 0, 0, DrawingPanel.this);
			System.out.println("paint");	
		}
		}catch(Exception e) {
			canvasZoomValue = 1.0f;
		}
		//setFocusToTextArea();
	}
		
	public int print(Graphics g, PageFormat pf, int page)
		throws PrinterException {
		if(page>=1) return Printable.NO_SUCH_PAGE;
		Graphics2D g2 = (Graphics2D)g;
		g2.translate(pf.getImageableX(), pf.getImageableY());
	    if(antiMode) {
		    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
		          RenderingHints.VALUE_ANTIALIAS_ON);
	    }
	    //if(renderMode) {
	    	g2.setRenderingHint(RenderingHints.KEY_RENDERING,
	    			RenderingHints.VALUE_RENDER_QUALITY);
	    //}
		if(scalePrintFlag) {
			g2.scale(scalePrint, scalePrint);
		}
		boolean isZoom = isCanvasZoom;
		isCanvasZoom = false;
		//g2.draw(new Rectangle2D.Double(0, 0, pf.getImageableWidth(), 
		//pf.getImageableHeight()) );
		paint(g2);
		isCanvasZoom = isZoom;
		return 0;
	}
		
	public int getScrollableUnitIncrement(Rectangle visibleRect,
            int orientation,
            int direction) {		
		//System.out.println(visibleRect.width + " " + visibleRect.height);
		//System.out.println(orientation + " " + direction);
		return 15;		
	}
		
	public Dimension getPreferredScrollableViewportSize() {
        return getPreferredSize();
	}
		
	public int getScrollableBlockIncrement(Rectangle arg0, int arg1, int arg2) {
		return 250;
	}
		
	public boolean getScrollableTracksViewportWidth() { return false; }
		
	public boolean getScrollableTracksViewportHeight() { return false; }
		
	public int getElementNum() {
		return elementNum;
	}
	
	public int getLayerNum() {
		return layerNum;
	}
	
	public int getParentLayerNum() {
		return parentLayerNum;
	}
	
	public void setElementNum(int elementNum) {
		this.elementNum = elementNum;
	}
		
	public void addElementNum() {
		elementNum++;
	}
	
	public void setLayerNum(int layerNum) {
		this.layerNum = layerNum;
	}
	
	public void setParentLayerNum(int parentLayerNum) {
		this.parentLayerNum = parentLayerNum;
	}

	public int getParentLayerDepth() {
		return parentLayerDepth;
	}
	
	public void setParentLayerDepth(int parentLayerDepth) {
		this.parentLayerDepth = parentLayerDepth;
	}
	
	public int getCurrentElementIndex() {
		return currentElementIndex;
	}
		
	public boolean isLayerSelection() {
		return layerSelection;
	}
	
	public void setLayerSelection(boolean layerSelect) {
		this.layerSelection = layerSelect;
	}
	
	public boolean isColorMode() {
		return colorMode;
	}
	
	public boolean isStrokeMode() {
		return strokeMode;
	}

	public float getCanvasZoomValue() {
		return canvasZoomValue;
	}

	public float getPanelHeight() {
		return panelHeight;
	}

	public float getPanelWidth() {
		return panelWidth;
	}

	public boolean isGradientMode() {
		return gradientMode;
	}

	public boolean isPatternMode() {
		return patternMode;
	}

	public void setGradientMode(boolean gradientMode) {
		this.gradientMode = gradientMode;
	}

	public void setPatternMode(boolean patternMode) {
		this.patternMode = patternMode;
	}

	public float getStrokeSize() {
		return strokeSize;
	}

	public float getRoundRectArcHeight() {
		return roundRectArcHeight;
	}

	public float getRoundRectArcWidth() {
		return roundRectArcWidth;
	}

	public void setRoundRectArcWidth(float roundRectArcWidth) {
		this.roundRectArcWidth = roundRectArcWidth;
		if(!layerSelection) {
			if(drawVector.size()>currentElementIndex
					&& drawVector.get(currentElementIndex).getToolName() == ROUNDRECT) {
				((RoundRectTool)drawVector.get(currentElementIndex)).setArcWidth(
						roundRectArcWidth);

				MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
				repaintAll();
			}
		}
	}
	
	public void setRoundRectArcHeight(float roundRectArcHeight) {
		this.roundRectArcHeight = roundRectArcHeight;
		if(!layerSelection) {
			if(drawVector.size()>currentElementIndex
					&& drawVector.get(currentElementIndex).getToolName() == ROUNDRECT) {
				((RoundRectTool)drawVector.get(currentElementIndex)).setArcHeight(
						roundRectArcHeight);

				MainWindow.getInstance().vectorLayerDialog.vectorToolBarRepaint();
				repaintAll();
			}
		}
	}

	public int getDrawingTool() {
		return drawingTool;
	}

	public VolatileImage getVimg() {
		return vimg;
	}

}
