package com.sharppaint.drawtools;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.Serializable;
import java.util.*;
import javax.swing.ImageIcon;

import com.sharppaint.MainWindow;

public abstract class Element implements Serializable {	//Seria-lization
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1636571944837195791L;
	/************************version 0.8***********************/
	//layer Number
	protected int layerNum;
	//element Number in Layer
	protected int elementNum;
	//parent Layer Number
	protected int parentLayerNum;
	//parent Layer Depth
	protected int parentLayerDepth;
	//Layer Folding
	protected boolean layerFolding;
	
	
	//child Layer Number in Parent Layer
	//protected int positionInParent;
	protected float ctrlX1, ctrlY1;
	protected float ctrlX2, ctrlY2;
	
	/************************version 0.8***********************/
	/********************** never change *********************/
	transient protected Shape drawShape;
	transient protected Shape selectShape;
	transient protected Image image, patternImg, strokeImg, originalImg;
	transient protected GeneralPath path;
	transient protected boolean match, select;
	//name
	protected int toolName;
	//coordinate
	protected float x1, y1, x2, y2;
	//width, height
	protected float width, height;
	//pattern image's path
	protected String patternPath;
	//stroke pattern image's path
	protected String strokePatternPath;
	protected boolean hide;
	//color
	protected Color fillColor;
	protected Color strokeColor;
	//mode select variable 
	protected boolean colorMode;
	protected boolean strokeMode;
	protected boolean gradientMode;
	protected boolean patternMode;
	//opacity
	//protected float groupOpacity=1.0f;
	protected float colorOpacity=1.0f;
	protected float gradientOpacity=1.0f;
	protected float patternOpacity =1.0f;
	protected float strokeOpacity =1.0f;
	//	use setClip
	protected int screenWidth, screenHeight;		
	//edit
	protected int angle;
	protected double scaleX=1.0, scaleY=1.0;
	protected boolean rotateMode, scaleMode;
	//pattern
	protected float patternScale=1.0f;
	protected int patternScaleWidth;
	protected int patternSrcWidth, patternSrcHeight;
	//stroke
	protected float strokeSize;
	protected float strokePatternScale=1.0f;
	protected float strokeGradAngle=0.0f;
	protected float strokeGradScale=1.0f;
	protected int strokeFillSelector;		//color, grad, pattern mode
	protected int strokePosition;		//inside, center, outside mode
	protected int strokeImgWidth,	strokeImgHeight;
	protected int strokePatternScaleWidth;
	protected int strokeGradStyle;
	protected boolean isStrokeGradCycle = false; 
	protected float strokeGradRadius;
	
	//gradient
	protected int gradListSize;
	protected int strokeGradListSize;
	protected Color[] gradColors;
	protected float[] gradDist;
	protected Color[] strokeGradColors;
	protected float[] strokeGradDist;
	protected float scaleGrad=1.0f;
	protected float angleGrad=0.0f;
	transient protected GradientTool gradRect;
	protected int gradStyle;
	protected boolean isGradCycle = false;  
	protected float gradRadius;
	
	protected ArrayList<GradientTool> gradList, strokeGradList;
	private String strokeGradID;
	private String fillGradID;
	//multi line
	protected PointF[] points;
	protected int pointCount = 0;
	
	protected boolean isTransformPattern;
	protected boolean alreadyAntiAlias;
	
	transient protected AffineTransform at, saveform;
	transient protected AffineTransform gradAt, gradSaveForm;
	transient protected int rgbValue;
	transient protected Stroke selectStroke;
	transient protected BufferedImage bi;
	transient protected TexturePaint texture, strokeTexture;
	transient protected Graphics2D big;
	transient protected boolean firstPatternMode, firstStrokePatternMode;
	transient protected boolean reverseWidthFlag, reverseHeightFlag;
	/********************** never change *********************/
	
	Element() {
		Dimension d = MainWindow.getInstance().getPanel().getPanelSize();
		screenWidth = d.width;
		screenHeight = d.height;
		firstPatternMode = true;
		firstStrokePatternMode = true;
		strokeSize = 1.0f;
		strokeFillSelector = DrawToolConst.STROKE_COLOR;
		strokePosition = DrawToolConst.STROKE_POSITION_CENTER;
		selectStroke = new BasicStroke(1);
		//selectStroke = new BasicStroke(3, BasicStroke.CAP_SQUARE, 
        //        BasicStroke.JOIN_ROUND, 0, new float[]{0,6,0,6}, 0);
		//idGradList = new ArrayList<ArrayList<GradientTool>>();
		parentLayerNum = 0;
		parentLayerDepth = 0;
		layerFolding = false;
	}

	//edit (transform)
	public void setRotateAngle(int _angle) {
		angle = _angle;
		if(angle!=0)
			rotateMode = true;
		else
			rotateMode = false;
	}
	
	public void setScaleX(double _scaleX) {
		scaleX = _scaleX;
		if(scaleX == 1.0 && scaleY == 1.0)
			scaleMode = false;
		else
			scaleMode = true;
	}
	
	public void setScaleY(double _scaleY) {
		scaleY = _scaleY;
		if(scaleX == 1.0 && scaleY == 1.0)
			scaleMode = false;
		else
			scaleMode = true;
	}
	
	public void setRotateAndScale(
			double _scaleX, double _scaleY, int _angle) {
		scaleX = _scaleX;
		scaleY = _scaleY;
		angle = _angle;
		if(scaleX == 1.0 && scaleY == 1.0)
			scaleMode = false;
		else
			scaleMode = true;
		if(angle!=0)
			rotateMode = true;
		else
			rotateMode = false;
	}
	
	public double getScaleX() {
		return scaleX;
	}
	
	public double getScaleY() {
		return scaleY;
	}
	
	public int getRotateAngle() {
		return angle;
	}

	public void setRotateMode(boolean rotate) {
		this.rotateMode = rotate;
	}
	
	public boolean getScaleMode() {
		return scaleMode;
	}
	
	public void setScaleMode(boolean resize) {
		this.scaleMode = resize;
	}
	
	public boolean getTransformMode() {
		if(rotateMode==true || scaleMode==true)
			return true;
		else
			return false;
	}
	public void setColorMode(boolean _colorMode){ 
		colorMode = _colorMode;
	}
	
	public void setGradientMode(boolean _gradientMode){
		gradientMode = _gradientMode;
	}
	
	public void setPatternMode(boolean _patternMode){
		patternMode= _patternMode;
	}
	
	public void setStrokeMode(boolean _strokeMode) { 
		strokeMode = _strokeMode;
	}	
		
	public boolean getColorMode(){
		return colorMode;
	}
	
	public boolean getGradientMode(){
		return gradientMode;
	}
	
	public boolean getPatternMode(){
		return patternMode;
	}
	
	public boolean getStrokeMode(){
		return strokeMode;
	}
	
	//color
	public void setFillColor(Color _fillColor, int _colorOpacity, boolean _colorMode) {
		fillColor = _fillColor;
		colorOpacity = _colorOpacity / 100.0f;
		colorMode = _colorMode;
	}
	public void setFillColorFromSVG(Color _fillColor, float _colorOpacity) {
		fillColor = _fillColor;
		colorOpacity = _colorOpacity;
		colorMode = true;
	}

	public void setFillColor(Color _fillColor) {
		fillColor = _fillColor;
		colorMode = true;
	}
	
	public void setColorOpacityPercent(int _colorOpacity) {
		colorOpacity = _colorOpacity / 100.0f;
	}
	
	public void setColorOpacity(float _colorOpacity) {
		colorOpacity = _colorOpacity;
	}
		
	public float getColorOpacityPercent() {
		return colorOpacity*100;
	}

	public float getColorOpacity() {
		return colorOpacity;
	}
	
	public Color getFillColor() {
		return fillColor;		
	}
	
	//pattern
	public void setPatternImage(String path, int _patternScale, 
			int _patternOpacity, boolean _patternMode) {
		patternPath = path;
		patternImg = new ImageIcon(
				getClass().getResource(patternPath)).getImage();
		patternMode = _patternMode;
		patternSrcWidth = patternImg.getWidth(null);
		patternSrcHeight = patternImg.getHeight(null);
		patternOpacity = _patternOpacity / 100.0f;
		patternScale = _patternScale / 100.0f;
		patternScaleWidth = Math.round(patternSrcWidth * patternScale);
		firstPatternMode = true;
	}
		
	public void setPatternScale(int _patternScale) {
		patternScale = _patternScale / 100.0f;
		patternScaleWidth = Math.round(patternSrcWidth * patternScale);
		firstPatternMode = true;
	}
	
	public void setPatternOpacityPercent(int _patternOpacity) {
		patternOpacity = _patternOpacity / 100.0f;
		firstPatternMode = true;
	}
	
	public void setPatternOpacity(float _patternOpacity) {
		patternOpacity = _patternOpacity;
		firstPatternMode = true;
	}
	
	public float getPatternOpacityPercent() {
		return patternOpacity*100;
	}
	
	public float getPatternOpacity() {
		return patternOpacity;
	}
	
	public float getPatternScalePercent() {
		return patternScale*100;
	}
	
	public String getPatternPath() {
		return patternPath;
	}
	
	public void setFirstPatternMode() {
		firstPatternMode = true;
		firstStrokePatternMode = true;
	}
	
	//gradient
	public void setGradOpacityPercent(int _gradOpacity) {
		gradientOpacity = _gradOpacity / 100.0f;
	}
	
	public void setGradOpacity(float _gradOpacity) {
		gradientOpacity = _gradOpacity;
	}
	
	public ArrayList<GradientTool> getGradList() {
		return gradList;
	}
	
	public int getGradOpacityPercent() {
		return Math.round(gradientOpacity*100);
	}
	
	public float getGradOpacity() {
		return gradientOpacity;
	}
	
	public void setGradVar(float _scaleGrad, float _angleGrad) {
		scaleGrad = _scaleGrad/100.0f;
		angleGrad = _angleGrad;		
	}
	
	public void setGradVarFromSVG(float _scaleGrad, float _angleGrad) {
		scaleGrad = _scaleGrad;
		angleGrad = _angleGrad;		
	}
	
	public void setGradScalePercent(float _scaleGrad) {
		scaleGrad = _scaleGrad/100.0f;
	}
	
	public void setGradScale(float _scaleGrad) {
		scaleGrad = _scaleGrad;
	}
	
	public void setGradAngle(float _angleGrad) {
		angleGrad = _angleGrad;	
	}
	
	public float getGradScalePercent() {
		return scaleGrad*100;
	}
	
	public float getGradScale() {
		return scaleGrad;
	}
	
	public float getGradAngle() {
		return angleGrad;		
	}
	
	public float getGradientAngle360() {
		if(angleGrad<0)	{		
			return 360+angleGrad;
		} else {
			return angleGrad;
		}
	}
	
	public void setGradientFromSVG(ArrayList<GradientTool> _gradList) {
		gradList = new ArrayList<GradientTool>();
		for(int i=0; i<_gradList.size(); i++) {
			gradRect = new GradientTool(_gradList.get(i));
			gradList.add(gradRect);
		}
		gradientMode = true;
		initGradient();
	}
	
	public void setGradient(ArrayList<GradientTool> _gradList, 
			int _gradOpacity, boolean _gradientMode) {
		gradList = new ArrayList<GradientTool>();
		for(int i=0; i<_gradList.size(); i++) {
			gradRect = new GradientTool(_gradList.get(i));
			gradList.add(gradRect);
		}
		gradientMode = _gradientMode;
		gradientOpacity = _gradOpacity / 100.0f;
		initGradient();
	}
	protected void initGradient() {}
	protected void initStrokeGradient() {}
	//stroke
	public void setStrokeSize(float _strokeSize) {
		strokeSize = _strokeSize;
	}
	
	public float getStrokeSize() {
		return strokeSize;
	}
	
	public float getStrokePatternScalePercent() {
		return strokePatternScale*100;
	}
	
	public String getStrokePatternPath() {
		return strokePatternPath;
	}
	
	public void setStroke(float _strokeSize,  int _strokeOpacity, 
			int _strokePosition, int _strokeFillMode, boolean _strokeMode) {
		strokeSize = _strokeSize;
		strokeOpacity = _strokeOpacity / 100.0f;
		strokePosition = _strokePosition;
		strokeFillSelector = _strokeFillMode;
		strokeMode = _strokeMode;
	}
	
	public void setStrokeFromSVG(float _strokeSize,  float _strokeOpacity, 
			int _strokePosition, int _strokeFillMode) {
		strokeSize = _strokeSize;
		strokeOpacity = _strokeOpacity;
		strokePosition = _strokePosition;
		strokeFillSelector = _strokeFillMode;
		strokeMode = true;
	}
	
	public void setStrokePatternImage(String path, int _strokePatternScale) {
		strokePatternPath = path;
		strokeImg = new ImageIcon(
				getClass().getResource(strokePatternPath)).getImage();
		strokeFillSelector = DrawToolConst.STROKE_PATTERN;
		strokeImgWidth = strokeImg.getWidth(null);
		strokeImgHeight = strokeImg.getHeight(null);
		strokePatternScale = _strokePatternScale / 100.0f;
		strokePatternScaleWidth = Math.round(strokeImgWidth * strokePatternScale);
		firstStrokePatternMode = true;
	}
	
	public void setStrokePatternScale(int _strokePatternScale) {
		strokePatternScale = _strokePatternScale / 100.0f;
		strokePatternScaleWidth = Math.round(strokeImgWidth * strokePatternScale);
		firstStrokePatternMode = true;
	}
	
	public void setStrokeOpacityPercent(int _strokeOpacity) {
		strokeOpacity = _strokeOpacity / 100.0f;
	}
	
	public void setStrokeOpacity(float _strokeOpacity) {
		strokeOpacity = _strokeOpacity;
	}
	
	public float getStrokeOpacityPercent() {
		return strokeOpacity*100;
	}
	
	public float getStrokeOpacity() {
		return strokeOpacity;
	}
	
	public void setStrokeFillMode(int _strokeFillMode) {
		strokeFillSelector = _strokeFillMode;
	}
	
	public void setStrokePosition(int _strokePosition) {
		strokePosition = _strokePosition;
	}
	
	public void setStrokeGradAngle(float _angleGrad) {
		strokeGradAngle = _angleGrad;		
	}
		
	public void setStrokeGradScale(float _scaleGrad) {
		strokeGradScale = _scaleGrad/100.0f;		
	}
	
	public void setStrokeColor(Color _strokeColor) {
		strokeFillSelector = DrawToolConst.STROKE_COLOR;
		strokeColor = _strokeColor;
		strokeMode = true;
	}
	
	public Color getStrokeColor() {
		return strokeColor;		
	}
	
	public int getStrokeGradScalePercent() {
		return Math.round(strokeGradScale*100);
	}
	
	public float getStrokeGradScale() {
		return strokeGradScale;
	}
	
	public float getStrokeGradAngle() {
		return strokeGradAngle;		
	}

	public float getStrokeGradientAngle360() {
		if(strokeGradAngle<0)
			return 360+strokeGradAngle;
		else
			return strokeGradAngle;
	}
	
	public void setStrokeGradient(ArrayList<GradientTool> _sgradList) {
		strokeFillSelector = DrawToolConst.STROKE_GRADIENT;
		strokeGradList = new ArrayList<GradientTool>();
		for(int i=0; i<_sgradList.size(); i++) {
			gradRect = new GradientTool(_sgradList.get(i));
			strokeGradList.add(gradRect);
		}
		initStrokeGradient();
	}
	
	public int getStrokeFillMode() {
		return strokeFillSelector;
	}
	
	public int getStrokePosition() {
		return strokePosition;
	}
	
	public ArrayList<GradientTool> getStrokeGradList() {
		return strokeGradList;/*		if(strokeGradList!=null)
		return (ArrayList<GradientTool>)strokeGradList.clone();else	return null;*/
	}	
	
	public boolean getIsGradList() {
		if(gradList != null)
			return true;
		else
			return false;
	}
		
	public int getToolName() {
		return toolName;
	}
	
	public void setHide(boolean _hide) {
		hide = _hide;
	}
	
	public boolean getHide() {
		return hide;
	}
	
	public void setEqualWidthHeight() {
		if(width<height) {
			height = width;
		}
		else {
			width = height;
		}
		
		if(reverseWidthFlag) {
			x1 = x2 - width;
		}
		if(reverseHeightFlag) {
			y1 = y2 - height;
		}
		//System.out.println(x1 + " " + y1 + " " + x2 + " " +y2);
	}	
	
	public void setValue(float _x1, float _y1) {
		x1=_x1;
		y1=_y1;
	}
	
	public void setValue(float _x1, float _y1, float _x2, float _y2) {
		x1=_x1;
		y1=_y1;
		x2=_x2;
		y2=_y2;
		width = x2 - x1;
		height = y2 - y1;
		//System.out.println(width);
		//System.out.println(height);
	}
	
	public void setTransValue(float _x1, float _y1, float _x2, float _y2) {
		if(_x1 < _x2){ 
			x1 = _x1;
			x2 = _x2;
			reverseWidthFlag = false;
		}
		else{ //   ų   ٲ 
			x1 = _x2;
			x2 = _x1;
			reverseWidthFlag = true;
		}
		if(_y1 < _y2){ 
			y1 = _y1; 
			y2 = _y2;
			reverseHeightFlag = false;
		}
		else{ //   ų   ٲ 
			y1 = _y2;
			y2 = _y1;
			reverseHeightFlag = true;
		}
		width = x2 - x1;
		height = y2 - y1;
	}
	
	public void resize(float _x1, float _y1, float _x2, float _y2) {	}
	
	abstract public void translate(
			float _x1, float _y1, float _x2, float _y2);
	
	abstract public boolean match(float x,float y);
	
	abstract public void draw(Graphics g);
	
	public void smallDraw(Graphics g, float scale) {}
	
	public void simpleDraw(Graphics g, float scale) {}	
	
	public void setSelected(boolean sel) {
		select = sel;
		reShape = false;
	}
	
	public float getX1(){return x1;}
	
	public float getY1(){return y1;}
	
	public float getX2(){return x2;}
	
	public float getY2(){return y2;}
	
	public float getWidth(){return width;}
	
	public float getHeight(){return height;}
	
	public void setX1(float _x1){x1 = _x1;}
	
	public void setY1(float _y1){y1 = _y1;}
	
	public void setX2(float _x2){x2 = _x2;}
	
	public void setY2(float _y2){y2 = _y2;}
	
	public void setWidth(float _width){ 
		width = _width;
		x2 = x1 + width;
	}
	
	public void setHeight(float _height){
		height = _height;
		y2 = y1 + height;
	}		

	public void setPatternImg(Image _patternImg) {
		patternImg = _patternImg;
		patternSrcWidth = patternImg.getWidth(null);
		patternSrcHeight = patternImg.getHeight(null);
		firstPatternMode = true;
	}
	
	public void setStrokePatternImage(Image _strokeImg) {
		strokeImg  = _strokeImg;
		strokeImgWidth = strokeImg.getWidth(null);
		strokeImgHeight = strokeImg.getHeight(null);
		firstStrokePatternMode = true;
	}
	
	public Image getPatternImg() {
		return patternImg;
	}
	
	public Image getStrokePatternImg() {
		return strokeImg;
	}
	
	public int getPatternImgWidth() {
		return patternSrcWidth; 
	}
	
	public int getPatternImgHeight() {
		return patternSrcHeight; 
	}
	
	public int getSPatternImgWidth() {
		return strokeImgWidth; 
	}
	
	public int getSPatternImgHeight() {
		return strokeImgHeight; 
	}
	
	public boolean getRotateMode() {
		return rotateMode;
	}	
	public int getElementNum() {
		return elementNum;
	}

	public int getLayerNum() {
		return layerNum;
	}

	public int getParentLayerNum() {
		return parentLayerNum;
	}

	public void setElementNum(int elementNum) {
		this.elementNum = elementNum;
	}
	
	public void addElementNum() {
		elementNum++;
	}
	
	public void addElementNum(int addCount) {
		elementNum = elementNum+addCount;
	}
	
	public void subElementNum() {
		elementNum--;
	}
	
	public void subElementNum(int subCount) {
		elementNum = elementNum-subCount;
	}

	public void setLayerNum(int layerNum) {
		this.layerNum = layerNum;
	}

	public void setParentLayerNum(int parentLayerNum) {
		this.parentLayerNum = parentLayerNum;
	}

	public int getParentLayerDepth() {
		return parentLayerDepth;
	}

	public void setParentLayerDepth(int parentLayerDepth) {
		this.parentLayerDepth = parentLayerDepth;
	}
	
	public void addParentLayerDepth(int interval) {
		parentLayerDepth = parentLayerDepth + interval;
	}
	
	public void subParentLayerDepth() {
		parentLayerDepth--;
	}

	public boolean isLayerFolding() {
		return layerFolding;
	}

	public void setLayerFolding(boolean layerFolding) {
		this.layerFolding = layerFolding;
	}
	
	public String getStrokeGradID() {
		return strokeGradID;
	}
	
	public void setStrokeGradID(String GradID) {
		this.strokeGradID = GradID;
	}	
	
	public String getFillGradID() {
			return fillGradID;
	}
	
	public void setFillGradID(String gradID) {
		this.fillGradID = gradID;
	}

	public int getPointCount() {
		return pointCount;
	}
	
	public void setGradStyle(int gradStyle) {
		this.gradStyle = gradStyle;
	}
	
	public void setStrokeGradStyle(int strokeGradStyle) {
		this.strokeGradStyle = strokeGradStyle;		
	}
	
	public int getGradStyle() {
		return gradStyle;
	}
	
	public int getStrokeGradStyle() {
		return strokeGradStyle;
	}
	
	public boolean isGradCycle() {
		return isGradCycle;
	}

	public void setGradCycle(boolean isRepeat) {
		this.isGradCycle = isRepeat;
	}
	
	public boolean isStrokeGradCycle() {
		return isStrokeGradCycle;
	}

	public void setStrokeGradCycle(boolean isStrokeGradCycle) {
		this.isStrokeGradCycle = isStrokeGradCycle;
	}

	public float getGradRadius() {
		return gradRadius;
	}

	public float getStrokeGradRadius() {
		return strokeGradRadius;
	}

	public void setGradRadius(float gradRadius) {
		//gradRadius = scaleGrad*width/2.0f;
		scaleGrad = gradRadius / width * 2.0f;
		//by Anne
		//float temp = ((100/(width/gradRadius))/100)*2;
		this.gradRadius = gradRadius;
	}
	
	/*
	public void calcGradScaleFromRadius() {
		
	}*/

	public void setStrokeGradRadius(float strokeGradRadius) {
		strokeGradScale = strokeGradRadius / width * 2.0f;
		this.strokeGradRadius = strokeGradRadius;
	}

	public boolean reShapeMatch(float x, float y) {
		initSelectStroke();
		reShapeMatch = false;
		for(int reShapeMatchIndex=0; reShapeMatchIndex<8; reShapeMatchIndex++) {
			if(reShapeList.get(reShapeMatchIndex).contains(x, y)) {
				reShapeMatch = true;
				return true;
				//break;
			}
		}
		return false;
	}
	boolean reShapeMatch;
	int reShapeMatchIndex = 0;
	int SELECT_RECT_SIZE = 4;
	transient protected Shape reShape1;
	transient protected Shape reShape2;
	transient protected Shape reShape3;
	transient protected Shape reShape4;
	transient protected Shape reShape5;
	transient protected Shape reShape6;
	transient protected Shape reShape7;
	transient protected Shape reShape8;
	//transient protected Shape tempShape;
	transient boolean reShape;
	transient protected ArrayList<Shape> reShapeList;
	protected void initSelectStroke() {
		reShapeList = new ArrayList<Shape>();
		reShape1 = new Rectangle2D.Float(x1-2, y1-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);
		reShapeList.add(reShape1);
		reShape2 = new Rectangle2D.Float((x1+width/2)-2, y1-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);
		reShapeList.add(reShape2);
		reShape3 = new Rectangle2D.Float((x1+width)-2, y1-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);
		reShapeList.add(reShape3);
		reShape4 = new Rectangle2D.Float(x1-2, y1+height/2-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);
		reShapeList.add(reShape4);
		reShape5 = new Rectangle2D.Float(x1+width-2, y1+height/2-2, 
		SELECT_RECT_SIZE, SELECT_RECT_SIZE);
		reShapeList.add(reShape5);
		reShape6 = new Rectangle2D.Float(x1-2,(y1+height)-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);
		reShapeList.add(reShape6);
		reShape7 = new Rectangle2D.Float(x1+width/2-2, y1+height-2, 
		SELECT_RECT_SIZE, SELECT_RECT_SIZE);
		reShapeList.add(reShape7);
		reShape8 = new Rectangle2D.Float(x1+width-2, y1+height-2, 
		SELECT_RECT_SIZE, SELECT_RECT_SIZE);
		reShapeList.add(reShape8);
	}
	protected void drawReshapeStroke(Graphics2D g2d, boolean transformMode) {
		initSelectStroke();
		if(!transformMode)
			selectShape = new Rectangle2D.Float(x1, y1, width, height);
		else
			selectShape = new Rectangle2D.Float(
					-width/2, -height/2, width, height);
		if(selectStroke == null)
			selectStroke =  new BasicStroke(1);
		g2d.setStroke(selectStroke);
		g2d.setColor(Color.BLUE);					
		g2d.draw(selectShape);
		
		g2d.setColor(Color.WHITE);
		g2d.fill(reShape1);
		g2d.fill(reShape2);
		g2d.fill(reShape3);
		g2d.fill(reShape4);
		g2d.fill(reShape5);
		g2d.fill(reShape6);
		g2d.fill(reShape7);
		g2d.fill(reShape8);
		g2d.setColor(Color.BLUE);	
		g2d.draw(reShape1);
		g2d.draw(reShape2);
		g2d.draw(reShape3);
		g2d.draw(reShape4);
		g2d.draw(reShape5);
		g2d.draw(reShape6);
		g2d.draw(reShape7);
		g2d.draw(reShape8);
		/*
		g2d.fillRect((int)x1-2, (int)y1-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);
		g2d.fillRect((int)(x1+width/2)-2, (int)y1-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);	
		g2d.fillRect((int)(x1+width)-2, (int)y1-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);	
		
		g2d.fillRect((int)x1-2, (int)(y1+height/2-2), 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);		
		g2d.fillRect((int)(x1+width-2), (int)(y1+height/2)-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);	
		
		g2d.fillRect((int)x1-2, (int)(y1+height)-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);		
		g2d.fillRect((int)(x1+width/2)-2, (int)(y1+height)-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);	
		g2d.fillRect((int)(x1+width)-2, (int)(y1+height)-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);	
		
		g2d.setColor(Color.BLUE);			
		g2d.drawRect((int)x1-2, (int)y1-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);		
		g2d.drawRect((int)(x1+width/2)-2, (int)y1-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);	
		g2d.drawRect((int)(x1+width-2), (int)y1-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);	
		
		g2d.drawRect((int)x1-2, (int)(y1+height/2)-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);		
		g2d.drawRect((int)(x1+width)-2, (int)(y1+height/2)-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);	
		
		g2d.drawRect((int)x1-2, (int)(y1+height)-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);		
		g2d.drawRect((int)(x1+width/2)-2, (int)(y1+height)-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);	
		g2d.drawRect((int)(x1+width)-2, (int)(y1+height)-2, 
				SELECT_RECT_SIZE, SELECT_RECT_SIZE);	
		*/
		//g2d.drawRect(x1-3, y2-3, 6, 6);
	}
	//protected void drawSelectStroke(Graphics2D g2d, boolean transformMode) {
		//g2d.setColor(Color.RED);
	//}
	/*
	initGradient();
	drawGradient(g2d);
	
	initStrokeGradient();
	drawStrokeGradient(g2d);
	
	drawSelectStroke(g2d);
	*/

	public boolean isReShape() {
		return reShape;
	}

	public void setReShape(boolean reShape) {
		select = false;
		this.reShape = reShape;
	}
	
}
