package com.sharppaint.drawtools;
import java.awt.*;
import java.awt.color.ColorSpace;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.*;
import java.util.ArrayList;
import javax.swing.ImageIcon;

public class ImageTool extends GradientHelper{
	/**
	 * 
	 */
	private static final long serialVersionUID = -2294899422006767217L;

	/********************** never change *********************/
	private boolean openImage;
	private boolean drawImage;
	private boolean grayScale, blur, sharpen, brighten, edge, negative;
	private int imageWidth, imageHeight;
	transient private float[] sharpenElements, blurElements, edgeElements;
	transient private byte negatives[];
	transient private boolean firstGrayScale, firstBlur;
	transient private boolean firstSharpen, firstBrighten;
	transient private boolean firstEdge, firstNegative;
	transient private BufferedImage bimage, grayImg, blurImg;
	transient private BufferedImage sharpenImg, brightenImg;
	transient private BufferedImage edgeImg, negativeImg;
	transient private ColorSpace cs;
	transient private ColorConvertOp theOp;
    transient private RenderingHints rhs;
	/********************** never change *********************/
    
	public ImageTool(float _x1, float _y1) {
		toolName = DrawToolConst.IMAGE;
		x1=_x1;
		y1=_y1;
		angle = 0;
		rotateMode = false;
		openImage = false;
		drawImage = false;	//grayScale = false;
	}
	public ImageTool(Element e) {
		toolName = DrawToolConst.IMAGE;
		x1=e.x1;
		y1=e.y1;
		x2=e.x2;
		y2=e.y2;
		image = e.originalImg;
		originalImg = e.originalImg;
		
		colorMode = e.colorMode;
		gradientMode = e.gradientMode;
		patternMode = e.patternMode;
		strokeMode = e.strokeMode;
		
		angle = e.angle;
		rotateMode = e.rotateMode;
		scaleMode = e.scaleMode;
		scaleX = e.scaleX;
		scaleY = e.scaleY;
		selectStroke = e.selectStroke;
		if(colorMode) {
			colorOpacity = e.colorOpacity;
			fillColor = e.fillColor;
		}
		if(gradientMode) {
			gradientOpacity = e.gradientOpacity;
			scaleGrad = e.scaleGrad;
			angleGrad = e.angleGrad;
			gradStyle = e.gradStyle;
			isGradCycle = e.isGradCycle;	
			gradList = new ArrayList<GradientTool>();
			for(int i=0; i<e.gradList.size(); i++) {
				gradRect = new GradientTool(e.gradList.get(i));
				gradList.add(gradRect);
			}
			initGradient();
		}
		if(patternMode) {
			patternPath = e.patternPath;
			patternScale = e.patternScale;
			patternOpacity = e.patternOpacity;
			patternImg = new ImageIcon(
					getClass().getResource(patternPath)).getImage();
			patternSrcWidth = e.patternSrcWidth;
			patternSrcHeight = e.patternSrcHeight;
			patternScaleWidth = e.patternScaleWidth;
		}
		if(strokeMode) {
			strokeSize = e.strokeSize;
			strokeOpacity = e.strokeOpacity;
			strokePosition = e.strokePosition;
			strokeFillSelector = e.strokeFillSelector;
			if(strokeFillSelector == 0) {
				strokeColor = e.strokeColor;
			}
			else if(strokeFillSelector == 1) {
				strokeGradScale = e.strokeGradScale;
				strokeGradAngle = e.strokeGradAngle;
				strokeGradStyle = e.strokeGradStyle;
				isStrokeGradCycle = e.isStrokeGradCycle;
				strokeGradList = new ArrayList<GradientTool>();
				for(int i=0; i<e.strokeGradList.size(); i++) {
					gradRect = new GradientTool(e.strokeGradList.get(i));
					strokeGradList.add(gradRect);
				}
				initStrokeGradient();
			}
			else if(strokeFillSelector == 2) {
				strokePatternScale = e.strokePatternScale;
				strokePatternPath = e.strokePatternPath;
				strokeImg = new ImageIcon(
						getClass().getResource(strokePatternPath)).getImage();
				strokeImgWidth = e.strokeImgWidth;
				strokeImgHeight = e.strokeImgHeight;
				strokePatternScaleWidth = e.strokePatternScaleWidth;
			}
		}
		drawImage = true;
		imageWidth = image.getWidth(null);
		imageHeight = image.getHeight(null);
		width = e.width;
		height = e.height;
		grayScale = ((ImageTool)e).grayScale;
		blur = ((ImageTool)e).blur;
		sharpen = ((ImageTool)e).sharpen;
		brighten = ((ImageTool)e).brighten;
		edge = ((ImageTool)e).edge;
		negative = ((ImageTool)e).negative;
		initElements();
		selectStroke = e.selectStroke;
	}
	public ImageTool(String path) {
		toolName = DrawToolConst.IMAGE;
		image = Toolkit.getDefaultToolkit().getImage(path);
		originalImg = Toolkit.getDefaultToolkit().getImage(path);
		openImage = true;
		imageWidth = image.getWidth(null);
		imageHeight = image.getHeight(null);
		width = imageWidth;
		height = imageHeight;
		x2=width;
		y2=height;	
		initElements();
	}
	public void initElements() {
		sharpenElements = new float[]{
			0.0f, -1.0f, 0.0f,
			-1.0f, 5.f, -1.0f,
			0.0f, -1.0f, 0.0f
		};
		float weight = 1.0f/9.0f;
		blurElements = new float[9];
		for(int i=0; i<9; i++) {
			blurElements[i]=weight;				
		}
		edgeElements = new float[]{
				/*
			0.0f, -1.0f, 0.0f,
			-1.0f, 4.f, -1.0f,
			0.0f, -1.0f, 0.0f*/
			-1.0f, -1.0f, -1.0f,
			0.0f, 0.0f, 0.0f,
			1.0f, 1.0f, 1.0f
		};	
		negatives = new byte[256];
		firstGrayScale = true;
		firstBlur = true;
		firstSharpen = true;
		firstBrighten = true;
		firstEdge = true;
		firstNegative = true;
	}
	
	public void openImage(String path) {
		image = Toolkit.getDefaultToolkit().getImage(path);
		originalImg = Toolkit.getDefaultToolkit().getImage(path);
		drawImage = true;
		imageWidth = image.getWidth(null);
		imageHeight = image.getHeight(null);
		width = x2-x1;
		height = y2-y1;
		initElements();
	}

	public void resize(float _x1, float _y1, float _x2, float _y2) {
		if(Math.abs(x1-_x1) < Math.abs(x2-_x1)){
			setTransValue( x1-(_x1-_x2), y1, x2, y2 );
		}
		else{
			setTransValue( x1, y1, x2-(_x1-_x2), y2 );
		}		
		if(Math.abs(y1-_y1) < Math.abs(y2-_y1)){
			setTransValue( x1, y1-(_y1-_y2), x2, y2 );
		}
		else{
			setTransValue( x1, y1, x2, y2-(_y1-_y2));
		}
	}
	
	public void setOriginalImageScale() {
		if(imageWidth != 0 && width != 0) {
			float temp = (float)imageWidth / (float)width;
			if(temp != 0.0f) {
				height = Math.round((float)imageHeight / temp);
				y2 = y1 + height;
			}
		}
	}
	
	public void setGrayScale(boolean _gray) {
		grayScale = _gray;
		firstGrayScale = true;
		firstBlur = true;
		firstSharpen = true;
		firstBrighten = true;
		firstEdge = true;
		firstNegative = true;
	}
	
	public boolean getGrayScale() {
		return grayScale;
	}
	
	public boolean getBlur() {
		return blur;
	}
	
	public boolean getSharpen() {
		return sharpen;
	}
	
	public boolean getBrighten() {
		return brighten;
	}
	
	public boolean getEdge() {
		return edge;
	}
	
	public boolean getNegative() {
		return negative;
	}
	
	public void setBlur(boolean _blur) {
		blur = _blur;
		firstBlur = true;
		firstSharpen = true;
		firstBrighten = true;
		firstEdge = true;
		firstNegative = true;
	}
	
	public void setSharpen(boolean _sharpen) {
		sharpen = _sharpen;
		firstSharpen = true;
		firstBrighten = true;
		firstEdge = true;
		firstNegative = true;
	}
	
	public void setBrighten(boolean _brighten) {
		brighten = _brighten;
		firstBrighten = true;
		firstEdge = true;
		firstNegative = true;
	}
	
	public void setEdge(boolean _edge) {
		edge = _edge;
		firstEdge = true;
		firstNegative = true;
	}
	
	public void setNegative(boolean _negative) {
		negative = _negative;
		firstNegative = true;
	}
	
	public void simpleDraw(Graphics g, float scale) {
		Graphics2D g2d = (Graphics2D) g;
		if(openImage==true) { 
			try {
				g2d.drawImage(image, 0, 0, 
						Math.round(image.getWidth(null)*scale), 
						Math.round(image.getHeight(null)*scale), 
						0, 0, 
						image.getWidth(null),
						image.getHeight(null), null);
			} catch(Exception e) {		}
		}
		else {
			AffineTransform ats = new AffineTransform();	
			AffineTransform saveforms = g2d.getTransform();
			ats.scale(scale, scale);
			g2d.transform(ats);
			g2d.setStroke(new BasicStroke(5));
			if(rotateMode || scaleMode) {
				at = new AffineTransform();	
				saveform = g2d.getTransform();		
				at.setToTranslation(x1 + width/2, y1 + height/2);
				if(rotateMode) { 
					at.rotate(Math.toRadians(angle)); 
				}
				if(scaleMode) {
					at.scale(scaleX, scaleY);
				}
				g2d.transform(at); 
				try {
					if(drawImage == true)
						g2d.drawImage(image, 
								Math.round(-width/2), 
								Math.round(-height/2), 
								Math.round(width/2), 
								Math.round(height/2), 
								0, 0, 
								image.getWidth(null), 
								image.getHeight(null), null);
				} catch(Exception e) {		}
				g2d.setTransform(saveform);
			}
			else {	//not rotate, not resize mode
				try {
					if(drawImage == true) {			
						g2d.drawImage(image, 
								Math.round(x1), Math.round(y1), 
								Math.round(x2), Math.round(y2), 
								0, 0, 
								image.getWidth(null), 
								image.getHeight(null), null);
					}
				} catch(Exception e) {		}
			}	//not rotate block end		
			g2d.setTransform(saveforms);
		}
	}
	public void smallDraw(Graphics g, float scale) {
		Graphics2D g2d = (Graphics2D) g;
		if(openImage==true) { 
			try {
				g2d.drawImage(image, 0, 0, 
						Math.round(image.getWidth(null)*scale), 
						Math.round(image.getHeight(null)*scale),
						0, 0, image.getWidth(null), 
						image.getHeight(null), null);
			} catch(Exception e) {		}
		}
		else {
			AffineTransform ats = new AffineTransform();	
			AffineTransform saveforms = g2d.getTransform();
			ats.scale(scale, scale);
			g2d.transform(ats);
			g2d.setStroke(new BasicStroke(5));
			if(rotateMode || scaleMode) {
				at = new AffineTransform();	
				saveform = g2d.getTransform();		
				at.setToTranslation(x1 + width/2, y1 + height/2);
				if(rotateMode) { 
					at.rotate(Math.toRadians(angle)); 
				}
				if(scaleMode) {
					at.scale(scaleX, scaleY);
				}
				g2d.transform(at); 
				try {
					if(drawImage == true)
						g2d.drawImage(image, 
								Math.round(-width/2), 
								Math.round(-height/2), 
								Math.round(width/2), 
								Math.round(height/2), 								
								0, 0, image.getWidth(null), 
								image.getHeight(null), null);
				} catch(Exception e) {		}
				g2d.setTransform(saveform);
			}
			else {	//not rotate, not resize mode
				try {
					if(drawImage == true) {			
						g2d.drawImage(image, 
								Math.round(x1), Math.round(y1), 
								Math.round(x2), Math.round(y2),  
								0, 0,
								image.getWidth(null), 
								image.getHeight(null), null);
					}
				} catch(Exception e) {		}
			}		
			g2d.setTransform(saveforms);
		}
	}
	public void draw(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;
		g2d.setRenderingHint(RenderingHints.KEY_RENDERING,
				RenderingHints.VALUE_RENDER_QUALITY);
		if(hide == false) {
			if(rotateMode || scaleMode) {
				at = new AffineTransform();	
				saveform = g2d.getTransform();		
				at.setToTranslation(x1 + width/2, y1 + height/2);
				drawShape = new Rectangle2D.Float(
						-width/2, -height/2, width, height);
				if(rotateMode) { 
					at.rotate(Math.toRadians(angle)); 
				}
				if(scaleMode) {
					at.scale(scaleX, scaleY);
				}
				g2d.transform(at);
			} else {
				drawShape = new Rectangle2D.Float(x1, y1, width, height);
			}
			try {
				AlphaComposite ac = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, 1.0f);
				g2d.setComposite(ac);
				if(drawImage == true || openImage == true) {
					if(grayScale == true) {
						if(firstGrayScale == true) {		
							bimage = new BufferedImage(
									Math.round(width), 
									Math.round(height), 	
									BufferedImage.TYPE_INT_RGB);		
					        Graphics2D srcG = bimage.createGraphics();
					        rhs = g2d.getRenderingHints();
					        srcG.setRenderingHints(rhs);
					        srcG.drawImage(image, 0, 0, 
									Math.round(width), 
									Math.round(height), 
					        		0, 0,
					        		image.getWidth(null), 
					        		image.getHeight(null), null);
					        cs = ColorSpace.getInstance(ColorSpace.CS_GRAY);
					        theOp = new ColorConvertOp(cs, rhs);
					        grayImg = new BufferedImage(
									Math.round(width), 
									Math.round(height), 
					        		BufferedImage.TYPE_INT_RGB);
					        theOp.filter(bimage, grayImg);
					        image = grayImg;   
					        firstGrayScale = false;
						}
						else
							image = grayImg; 
					}
					else {
						image = originalImg;
					}
					
					if(blur == true) {
						if(firstBlur == true) {
							blurImg = filter(blurElements, g2d);
							firstBlur = false;
						}
						else {
							image = blurImg;
						}
					}
					else {
						if(grayScale == true)
							image = grayImg;
						else
							image = originalImg;
					}
					
					if(sharpen == true) {
						if(firstSharpen == true) {
							sharpenImg = filter(sharpenElements, g2d);
							firstSharpen = false;
						}
						else {
							image = sharpenImg;
						}
					}
					else {
						if(blur == true)
							image = blurImg;
						else if(grayScale == true)
							image = grayImg;
						else
							image = originalImg;
					}
					
					if(brighten == true) {
						if(firstBrighten == true) {
							float a = 1.5f;
							float b = -20.0f;
							RescaleOp op = new RescaleOp(a, b, null);
							brightenImg = filter(op, g2d);
							firstBrighten = false;
						}
						else {
							image = brightenImg;
						}
					}
					else {
						if(sharpen == true)
							image = sharpenImg;
						else if(blur == true)
							image = blurImg;
						else if(grayScale == true)
							image = grayImg;
						else
							image = originalImg;
					}

					if(edge == true) {
						if(firstEdge == true) {
							edgeImg = filter(edgeElements, g2d);
							firstEdge = false;
						}
						else {
							image = edgeImg;
						}
					}
					else {
						if(brighten == true)
							image = brightenImg;
						else if(sharpen == true)
							image = sharpenImg;
						else if(blur == true)
							image = blurImg;
						else if(grayScale == true)
							image = grayImg;
						else
							image = originalImg;
					}
					
					if(negative == true) {
						if(firstNegative == true) {
							for(int i=0; i<256; i++)
								negatives[i] = (byte) (255-i);
							ByteLookupTable table = new ByteLookupTable(
									0, negatives);
							LookupOp op = new LookupOp(table, null);
							negativeImg = filter(op, g2d);
							firstNegative = false;
						}
						else {
							image = negativeImg;
						}
					}
					else {
						if(edge == true)
							image = edgeImg;
						else if(brighten == true)
							image = brightenImg;
						else if(sharpen == true)
							image = sharpenImg;
						else if(blur == true)
							image = blurImg;
						else if(grayScale == true)
							image = grayImg;
						else
							image = originalImg;
					}

					if(drawImage == true) {			
						if(rotateMode || scaleMode) {
							g2d.drawImage(image, 
									Math.round(-width/2), 
									Math.round(-height/2), 
									Math.round(width/2), 
									Math.round(height/2), 	
									0, 0, 
									image.getWidth(null), 
									image.getHeight(null), 
									null);
						} else {
							g2d.drawImage(image, 
									Math.round(x1), 
									Math.round(y1), 
									Math.round(x2), 
									Math.round(y2), 
									0, 0, 
									image.getWidth(null), 
									image.getHeight(null), null);   
						}
					}
					else if(openImage == true) {
						g2d.drawImage(image, 0, 0, null);
						width = image.getWidth(null);
						height = image.getHeight(null);
					}
				}
				else if(drawImage == false && openImage == false){
					g2d.setColor( Color.BLACK );
					g2d.drawRect( 
							Math.round(x1), 
							Math.round(y1),
							Math.round(width), 
							Math.round(height) );
				}	
			} catch(Exception e) {		}
			if(colorMode) {
				AlphaComposite ac = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, 
						colorOpacity);
				g2d.setComposite(ac);
				g2d.setColor(fillColor);
				g2d.fill(drawShape);
			}					
			if(gradientMode) {
				if(!rotateMode && !scaleMode)  {
					drawGradient(g2d);
				} else {
					drawGradientTransform(g2d);
				}
				g2d.fill(drawShape);
			}
			if(patternMode) {
				if(firstPatternMode) {
					bi = new BufferedImage(
							patternScaleWidth, patternScaleWidth, 
							BufferedImage.TYPE_INT_RGB);
					big = bi.createGraphics();
					big.drawImage(patternImg, 0, 0, 
							patternScaleWidth, patternScaleWidth, 
							0, 0, patternSrcWidth, patternSrcHeight, null);
					if(rotateMode || scaleMode) {
						texture = new TexturePaint(bi, 
								new Rectangle2D.Float(
										-width/2, -height/2, 
										patternScaleWidth, patternScaleWidth));
					} else {
						texture = new TexturePaint(bi, 
								new Rectangle2D.Float(x1, y1, 
										patternScaleWidth, patternScaleWidth));						
					}
					AlphaComposite pac = AlphaComposite.getInstance(
							AlphaComposite.SRC_OVER, 
							patternOpacity);
					g2d.setComposite(pac);
					g2d.setPaint(texture);						
					g2d.fill(drawShape);
					firstPatternMode = false;
				}
				else {
					AlphaComposite pac = AlphaComposite.getInstance(
							AlphaComposite.SRC_OVER, 
							patternOpacity);
					g2d.setComposite(pac);
					g2d.setPaint(texture);	
					g2d.fill(drawShape);
				}
			}
			if(strokeMode) {	//strokeMode
				g2d.setStroke(new BasicStroke(strokeSize));
				AlphaComposite ac = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, strokeOpacity);
				g2d.setComposite(ac);
				if(strokeFillSelector == 0) {	
					g2d.setColor(strokeColor);
				}
				else if(strokeFillSelector == 1) {	//gradient
					if(!rotateMode && !scaleMode)  {
						drawStrokeGradient(g2d);
					} else {
						drawStrokeGradientTransform(g2d);
					}
				}
				else if(strokeFillSelector == 2) {	//pattern
					if(firstStrokePatternMode) {
						BufferedImage bii = new BufferedImage(
								strokePatternScaleWidth, 
								strokePatternScaleWidth, 
								BufferedImage.TYPE_INT_RGB);
						Graphics2D biig = bii.createGraphics();
						biig.drawImage(strokeImg, 0, 0, strokePatternScaleWidth, 
								strokePatternScaleWidth, 0, 0, strokeImgWidth, 
								strokeImgHeight, null);
						if(rotateMode || scaleMode) {
							strokeTexture = new TexturePaint(
									bii, new Rectangle2D.Float(
									-width/2, -height/2,
									strokePatternScaleWidth, 
									strokePatternScaleWidth));
						} else {
							strokeTexture = new TexturePaint(
									bii, new Rectangle2D.Float(
									x1, y1, 
									strokePatternScaleWidth, 
									strokePatternScaleWidth));					
						}
						firstStrokePatternMode = false;
					}
					g2d.setPaint(strokeTexture);
					
				}				
				if(!rotateMode && !scaleMode)  {
					if(strokePosition == DrawToolConst.STROKE_POSITION_OUTSIDE) {
						g2d.draw(new Rectangle2D.Float(
								x1-strokeSize/2, y1-strokeSize/2, 
								width+strokeSize, height+strokeSize));
					}
					else if(strokePosition == DrawToolConst.STROKE_POSITION_INSIDE) {
						g2d.draw(new Rectangle2D.Float(
								x1+strokeSize/2, y1+strokeSize/2, 
								width-strokeSize, height-strokeSize));
					}
					else {	//center
						g2d.draw(drawShape);
					}
				} else {
					if(strokePosition == DrawToolConst.STROKE_POSITION_OUTSIDE) {
						g2d.draw(new Rectangle2D.Float(
								-width/2-strokeSize/2, 
								-height/2-strokeSize/2, 
								width+strokeSize, 
								height+strokeSize));
					}
					else if(strokePosition == DrawToolConst.STROKE_POSITION_INSIDE) {
						g2d.draw(new Rectangle2D.Float(
								-width/2+strokeSize/2, -height/2+strokeSize/2, 
								width-strokeSize, height-strokeSize));
					}
					else {	//center
						g2d.draw(drawShape);
					}					
				}
			}
			if(select) {
				if(selectStroke == null)
					selectStroke = new BasicStroke(1);
				g2d.setStroke(selectStroke);
				g2d.setColor(Color.BLUE);
				g2d.draw(drawShape);
			} else if(reShape) {
				drawReshapeStroke(g2d, rotateMode || scaleMode);				
			}
			if(rotateMode || scaleMode) {
				g2d.setTransform(saveform);
			}
		}
	}
	private BufferedImage filter(BufferedImageOp op, Graphics2D g2d) {
		bimage = new BufferedImage(
				Math.round(width), 
				Math.round(height), 
				BufferedImage.TYPE_INT_RGB);
	    Graphics2D srcG = bimage.createGraphics();
	    rhs = g2d.getRenderingHints();
	    srcG.setRenderingHints(rhs);
	    srcG.drawImage(image, 0, 0, 
				Math.round(width), 
				Math.round(height), 
	    		0, 0, image.getWidth(null), 
	    		image.getHeight(null), null);
		BufferedImage filteredImg = new BufferedImage(
				Math.round(width), 
				Math.round(height), 
				BufferedImage.TYPE_INT_RGB);
		op.filter(bimage, filteredImg);
		image = filteredImg;
		return filteredImg;	
	}
	private BufferedImage filter(float[] elements, Graphics2D g2d) {
		Kernel kernel = new Kernel(3, 3, elements);
		BufferedImageOp op = new ConvolveOp(kernel);
		bimage = new BufferedImage(
				Math.round(width), 
				Math.round(height), 
				BufferedImage.TYPE_INT_RGB);
	    Graphics2D srcG = bimage.createGraphics();
	    rhs = g2d.getRenderingHints();
	    srcG.setRenderingHints(rhs);
	    srcG.drawImage(image, 0, 0, 
				Math.round(width), 
				Math.round(height), 
	    		0, 0, image.getWidth(null), 
	    		image.getHeight(null), null);
		BufferedImage filteredImg = new BufferedImage(
				Math.round(width), 
				Math.round(height), 
				BufferedImage.TYPE_INT_RGB);
		op.filter(bimage, filteredImg);
		image = filteredImg;
		return filteredImg;
		//image = filteredImg;		
	}
	@Override
	public boolean match(float x, float y) {
		if ( (x>=x1 && x<=x2) && (y>=y1 && y<=y2) ) {
			match=true;
		}
		else
			match=false;
		return match;
	}
	public void translate(float _x1, float _y1, float _x2, float _y2) {
		setValue(x1-(_x1-_x2), y1-(_y1-_y2), x2-(_x1-_x2), y2-(_y1-_y2));
	}
	public Image getImage() {
		return originalImg;
	}
	public void setImage(Image _image) {
		image = _image;
		originalImg = _image;
	}
	public int getImageWidth() {
		return imageWidth; 
	}
	public int getImageHeight() {
		return imageHeight;
	}
}
