package com.sharppaint.drawtools;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import javax.swing.ImageIcon;

public class RectangleTool extends GradientHelper{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -644561131035655339L;
	
	/********************** never change *********************/
	public RectangleTool(float _x1, float _y1) {
		super();
		toolName = DrawToolConst.RECT;
		x1=_x1;
		y1=_y1;
		angle = 0;
		rotateMode = false;
	}
	
	public void setTextShape() {
		toolName = DrawToolConst.TEXT_EDIT_BOX;
	}
	
	public RectangleTool(Element e) {
		toolName = DrawToolConst.RECT;
		x1=e.x1;
		y1=e.y1;
		colorMode = e.colorMode;
		gradientMode = e.gradientMode;
		patternMode = e.patternMode;
		strokeMode = e.strokeMode;
		
		angle = e.angle;
		rotateMode = e.rotateMode;
		scaleMode = e.scaleMode;
		scaleX = e.scaleX;
		scaleY = e.scaleY;
		selectStroke = e.selectStroke;
		if(colorMode) {
			colorOpacity = e.colorOpacity;
			fillColor = e.fillColor;
		}
		if(gradientMode) {
			gradientOpacity = e.gradientOpacity;
			scaleGrad = e.scaleGrad;
			angleGrad = e.angleGrad;
			gradStyle = e.gradStyle;
			isGradCycle = e.isGradCycle;
			gradList = new ArrayList<GradientTool>();
			for(int i=0; i<e.gradList.size(); i++) {
				gradRect = new GradientTool(e.gradList.get(i));
				gradList.add(gradRect);
			}
			initGradient();
		}
		if(patternMode) {
			patternPath = e.patternPath;
			patternScale = e.patternScale;
			patternOpacity = e.patternOpacity;
			patternImg = new ImageIcon(
					getClass().getResource(patternPath)).getImage();
			patternSrcWidth = e.patternSrcWidth;
			patternSrcHeight = e.patternSrcHeight;
			patternScaleWidth = e.patternScaleWidth;
		}
		if(strokeMode) {
			strokeSize = e.strokeSize;
			strokeOpacity = e.strokeOpacity;
			strokePosition = e.strokePosition;
			strokeFillSelector = e.strokeFillSelector;
			if(strokeFillSelector == 0) {
				strokeColor = e.strokeColor;
			}
			else if(strokeFillSelector == 1) {
				strokeGradScale = e.strokeGradScale;
				strokeGradAngle = e.strokeGradAngle;
				strokeGradStyle = e.strokeGradStyle;
				isStrokeGradCycle = e.isStrokeGradCycle;
				strokeGradList = new ArrayList<GradientTool>();
				for(int i=0; i<e.strokeGradList.size(); i++) {
					gradRect = new GradientTool(e.strokeGradList.get(i));
					strokeGradList.add(gradRect);
				}
				initStrokeGradient();
			}
			else if(strokeFillSelector == 2) {
				strokePatternScale = e.strokePatternScale;
				strokePatternPath = e.strokePatternPath;
				strokeImg = e.strokeImg;
				strokeImg = new ImageIcon(
						getClass().getResource(strokePatternPath)).getImage();
				strokeImgWidth = e.strokeImgWidth;
				strokeImgHeight = e.strokeImgHeight;
				strokePatternScaleWidth = e.strokePatternScaleWidth;
			}
		}
		selectStroke = e.selectStroke;
	}

	public boolean match(float x, float y) {
		if ( (x>=x1 && x<=x2) && (y>=y1 && y<=y2) ) {
			match=true;
		}
		else
			match=false;

		/*
		if( rotateMode || scaleMode ){			
			Graphics2D g2d = (Graphics2D)MainWindow.getInstance().getPanel().getGraphics();
			at = new AffineTransform();		
			at.setToTranslation(x1 + width/2, y1 + height/2);
			g2d.transform(at);
			if(rotateMode) { 
				at.rotate(Math.toRadians(angle)); 
			}
			if(scaleMode) {
				at.scale(scaleX, scaleY);
			}
			if(drawShape.contains(x, y))
				match = true;
			else 
				match=false;
		} else {
			if(drawShape.contains(x, y))
				match = true;
			else 
				match=false;
			
		}*/
		return match;
	}
	
	public void translate(float _x1, float _y1, float _x2, float _y2) {
		setValue(x1-(_x1-_x2), y1-(_y1-_y2), x2-(_x1-_x2), y2-(_y1-_y2));
	}
	
	public void translate(float _x1, float _x2) {	//gradient paint 
		setValue(x1-(_x1-_x2), 0, x2-(_x1-_x2), 13);
	}
	
	public void resize(float _x1, float _y1, float _x2, float _y2) {
		if(Math.abs(x1-_x1) < Math.abs(x2-_x1)){
			setTransValue( x1-(_x1-_x2), y1, x2, y2 );
		} else {
			setTransValue( x1, y1, x2-(_x1-_x2), y2 );
		}		
		if(Math.abs(y1-_y1) < Math.abs(y2-_y1)){
			setTransValue( x1, y1-(_y1-_y2), x2, y2 );
		} else {
			setTransValue( x1, y1, x2, y2-(_y1-_y2));
		}
	}
	
	public void simpleDraw(Graphics g, float scale) {
		Graphics2D g2d = (Graphics2D) g;
		g2d.setColor( Color.WHITE );
		AffineTransform ats = new AffineTransform();	
		AffineTransform saveforms = g2d.getTransform();	
		//at.setToTranslation(x1 + width/2, y1 + height/2);
		ats.scale(scale, scale);
		g2d.transform(ats);
		g2d.setStroke(new BasicStroke(5));
		if( rotateMode || scaleMode ){
			at = new AffineTransform();	
			saveform = g2d.getTransform();		
			at.setToTranslation(x1 + width/2, y1 + height/2);
			drawShape = new Rectangle2D.Float(-width/2, -height/2, width, height);
			if(rotateMode) { 
				at.rotate(Math.toRadians(angle)); 
			}
			if(scaleMode) {
				at.scale(scaleX, scaleY);
			}
			g2d.transform(at); 
			g2d.fill(drawShape);
			g2d.setColor(Color.BLACK);
			g2d.draw(drawShape);
			g2d.setTransform(saveform);
		}
		else {	//not rotate, not resize mode
			drawShape = new Rectangle2D.Float(x1, y1, width, height);
			g2d.fill(drawShape);
			g2d.setColor(Color.BLACK);
			g2d.draw(drawShape);
		}	//not rotate block end		
		g2d.setTransform(saveforms);
	}
	
	public void draw(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;
		if(hide == false) {	
			if(rotateMode || scaleMode)  {
				at = new AffineTransform();	
				saveform = g2d.getTransform();		
				at.setToTranslation(x1 + width/2, y1 + height/2);
				drawShape = new Rectangle2D.Float(-width/2, -height/2, width, height);
				
				if(rotateMode) { 
					at.rotate(Math.toRadians(angle)); 
				}
				if(scaleMode) {
					at.scale(scaleX, scaleY);
				}
				g2d.transform(at); 
			} else {
				drawShape = new Rectangle2D.Float(x1, y1, width, height);	
			}
			
			if(colorMode) {
				AlphaComposite ac = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, colorOpacity);
				g2d.setComposite(ac);
				//System.out.println(colorOpacity + " f" +  fillColor);
				g2d.setColor(fillColor);
				g2d.fill(drawShape);
			}
			if(gradientMode)  {
				if(!rotateMode && !scaleMode)  {
					drawGradient(g2d);
				} else {
					drawGradientTransform(g2d);
				}
				g2d.fill(drawShape);
			}			
			if(patternMode) {
				if(firstPatternMode) {
					bi = new BufferedImage(patternScaleWidth, patternScaleWidth, 
							BufferedImage.TYPE_INT_RGB);
					big = bi.createGraphics();
					big.drawImage(patternImg, 0, 0, 
							patternScaleWidth, patternScaleWidth, 
							0, 0, patternSrcWidth, patternSrcHeight, null);
					if(rotateMode || scaleMode) {
						texture = new TexturePaint(bi, 
								new Rectangle2D.Float(-width/2, -height/2, 
										patternScaleWidth, patternScaleWidth));
					} else {
						texture = new TexturePaint(bi, 
								new Rectangle2D.Float(x1, y1, 
										patternScaleWidth, patternScaleWidth));						
					}
					
					AlphaComposite pac = AlphaComposite.getInstance(
							AlphaComposite.SRC_OVER, patternOpacity);
					g2d.setComposite(pac);
					g2d.setPaint(texture);						
					g2d.fill(drawShape);
					firstPatternMode = false;
				}
				else {
					AlphaComposite pac = AlphaComposite.getInstance(
							AlphaComposite.SRC_OVER, patternOpacity);
					g2d.setComposite(pac);
					g2d.setPaint(texture);
					g2d.fill(drawShape);
				}
			}
			if(strokeMode) {	//strokeMode
				g2d.setStroke(new BasicStroke(strokeSize));
				AlphaComposite ac = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, strokeOpacity);
				g2d.setComposite(ac);
				if(strokeFillSelector == 0) {	
					g2d.setColor(strokeColor);
				}
				else if(strokeFillSelector == 1) {	//gradient
					if(!rotateMode && !scaleMode)  {
						drawStrokeGradient(g2d);
					} else {
						drawStrokeGradientTransform(g2d);
					}
				}
				else if(strokeFillSelector == 2) {	//pattern
					if(firstStrokePatternMode) {
						BufferedImage bii = new BufferedImage(
								strokePatternScaleWidth, strokePatternScaleWidth, 
								BufferedImage.TYPE_INT_RGB);
						Graphics2D biig = bii.createGraphics();
						biig.drawImage(strokeImg, 0, 0, 
								strokePatternScaleWidth, strokePatternScaleWidth, 0, 0, 
								strokeImgWidth, strokeImgHeight, null);
						if(rotateMode || scaleMode) {
							strokeTexture = new TexturePaint(bii, 
									new Rectangle2D.Float(
											-width/2, -height/2,
											strokePatternScaleWidth, 
											strokePatternScaleWidth));
						} else {
							strokeTexture = new TexturePaint(bii, 
									new Rectangle2D.Float(
											x1, y1, 
											strokePatternScaleWidth, 
											strokePatternScaleWidth));					
						}
						firstStrokePatternMode = false;
					}
					g2d.setPaint(strokeTexture);
					
				}					
				if(!rotateMode && !scaleMode)  {
					if(strokePosition == DrawToolConst.STROKE_POSITION_OUTSIDE) {
						g2d.draw(new Rectangle2D.Float(
								x1-strokeSize/2, y1-strokeSize/2, 
								width+strokeSize, height+strokeSize));
					}
					else if(strokePosition == DrawToolConst.STROKE_POSITION_INSIDE) {
						g2d.draw(new Rectangle2D.Float(
								x1+strokeSize/2, y1+strokeSize/2, 
								width-strokeSize, height-strokeSize));
					}
					else {	//center
						g2d.draw(drawShape);
					}
				} else {
					if(strokePosition == DrawToolConst.STROKE_POSITION_OUTSIDE) {
						g2d.draw(new Rectangle2D.Float(
								-width/2-strokeSize/2, 
								-height/2-strokeSize/2, 
								width+strokeSize, 
								height+strokeSize));
					}
					else if(strokePosition == DrawToolConst.STROKE_POSITION_INSIDE) {
						g2d.draw(new Rectangle2D.Float(
								-width/2+strokeSize/2, -height/2+strokeSize/2, 
								width-strokeSize, height-strokeSize));
					}
					else {	//center
						g2d.draw(drawShape);
					}					
				}
			
			}
			if(select) {
				if(selectStroke == null)
					selectStroke = new BasicStroke(1);
				g2d.setStroke(selectStroke);
				g2d.setColor(Color.BLUE);
				g2d.draw(drawShape);
			} else if(reShape) {
				drawReshapeStroke(g2d, rotateMode || scaleMode);				
			}
			
			if(rotateMode==true || scaleMode==true)  {
				g2d.setTransform(saveform);
				//g2d.setClip(new Rectangle(0, 0, screenWidth, screenHeight));			
			}			
			//restore opacity
			//AlphaComposite ac = AlphaComposite.getInstance(
			//		AlphaComposite.SRC_OVER, 1.0f);
			//g2d.setComposite(ac);	
		}
	}
		
	public void smallDraw(Graphics g, float scale) {
		Graphics2D g2d = (Graphics2D) g;
		g2d.setColor( fillColor );
		AffineTransform ats = new AffineTransform();	
		AffineTransform saveforms = g2d.getTransform();	
		ats.scale(scale, scale);
		g2d.transform(ats);		
		if( rotateMode==true || scaleMode==true ) {	
			at = new AffineTransform();	
			saveform = g2d.getTransform();
			at.setToTranslation(x1 + width/2, y1 + height/2);
			drawShape = new Rectangle2D.Float(-width/2, -height/2, width, height);
			if(rotateMode) { 
				at.rotate(Math.toRadians(angle)); 
			}
			if(scaleMode) {
				at.scale(scaleX, scaleY);
			}
			g2d.transform(at); 
		} else {
			drawShape = new Rectangle2D.Float(x1, y1, width, height);
		}
		if(colorMode) {
			AlphaComposite ac = AlphaComposite.getInstance(
					AlphaComposite.SRC_OVER, colorOpacity);
			g2d.setComposite(ac);
			g2d.setColor(fillColor);
			g2d.fill(drawShape);
		}					
		if(gradientMode) {
			if(!rotateMode && !scaleMode)  {
				drawGradient(g2d);
			} else {
				drawGradientTransform(g2d);
			}
			g2d.fill(drawShape);
		}
		if(patternMode) {
			AlphaComposite pac = AlphaComposite.getInstance(
					AlphaComposite.SRC_OVER, patternOpacity);
			g2d.setComposite(pac);
			g2d.setPaint(texture);
			g2d.fill(drawShape);
		}
		if(strokeMode) {	//strokeMode
			g2d.setStroke(new BasicStroke(strokeSize));
			AlphaComposite ac = AlphaComposite.getInstance(
					AlphaComposite.SRC_OVER, strokeOpacity);
			g2d.setComposite(ac);
			if(strokeFillSelector == 0) {	
				g2d.setColor(strokeColor);
			}
			else if(strokeFillSelector == 1) {	//gradient
				if(!rotateMode && !scaleMode)  {
					drawStrokeGradient(g2d);
				} else {
					drawStrokeGradientTransform(g2d);
				}
			}
			else if(strokeFillSelector == 2) {	//pattern		
				g2d.setPaint(strokeTexture);				
			}								
			if(!rotateMode && !scaleMode)  {
				if(strokePosition == DrawToolConst.STROKE_POSITION_OUTSIDE) {	
					g2d.draw(new Rectangle2D.Float(
							x1-strokeSize/2, y1-strokeSize/2, 
							width+strokeSize, height+strokeSize));
				}
				else if(strokePosition == DrawToolConst.STROKE_POSITION_INSIDE) {
					g2d.draw(new Rectangle2D.Float(
							x1+strokeSize/2, y1+strokeSize/2, 
							width-strokeSize, height-strokeSize));
				}
				else {	//center
					g2d.draw(drawShape);
				}
			} else {
				if(strokePosition == DrawToolConst.STROKE_POSITION_OUTSIDE) {
					g2d.draw(new Rectangle2D.Float(
							-width/2-strokeSize/2, 
							-height/2-strokeSize/2, 
							width+strokeSize, 
							height+strokeSize));
				}
				else if(strokePosition == DrawToolConst.STROKE_POSITION_INSIDE) {	//inside
					g2d.draw(new Rectangle2D.Float(
							-width/2+strokeSize/2, -height/2+strokeSize/2, 
							width-strokeSize, height-strokeSize));
				}
				else {	//center
					g2d.draw(drawShape);
				}					
			}	
		}
		g2d.setTransform(saveforms);
	}
}